@@ -542,7 +542,7 @@ int	rams_run(char *mibSource, char *tsorder, char *applicationName,
 	socklen_t		nameLength;
 	int			datagramLength;
 	Lyst			msgspaces;
-	int			cId;
+	long			cId;
 	Petition		*pet;
 	AmsEventMgt		rules;
 	int			ownPseudoSubject;
@@ -621,7 +621,7 @@ printf("continuum lyst:");
 	{
 		cId = (long) lyst_data(elt);
 #if RAMSDEBUG
-printf(" %d", cId);		
+printf(" %ld", cId);		
 #endif
 		if (cId == gWay->amsMib->localContinuumNbr)
 		{
@@ -743,7 +743,7 @@ printf("bp_open succeeds.\n");
 		CHKERR(gWay->udpRpdus);
 		lyst_compare_set(gWay->udpRpdus, compareCheckTimes);
 		lyst_delete_set(gWay->udpRpdus, deleteDeclaration, NULL);
-		if (pthread_create(&checkThread, NULL, CheckUdpRpdus, NULL))
+		if (pthread_begin(&checkThread, NULL, CheckUdpRpdus, NULL))
 		{
 			putSysErrmsg("Can't create check thread", NULL);
 			return -1;
@@ -866,7 +866,7 @@ printf("Before bp_receive...\n");
 				break;
 
 			case BpPayloadPresent:
-				sdr_begin_xn(sdr);
+				CHKERR(sdr_begin_xn(sdr));
 				if (HandleBundle(&dlv, buffer) < 0)
 				{
 					sdr_cancel_xn(sdr);
@@ -931,7 +931,7 @@ printf("Before bp_receive...\n");
 	MRELEASE(buffer);		/*	Release RPDU buffer.	*/
 	if (gWay->netProtocol == RamsUdp)
 	{
-		pthread_cancel(checkThread);
+		pthread_end(checkThread);
 		pthread_join(checkThread, NULL);
 	}
 
@@ -1246,8 +1246,14 @@ static void	HandleUnregistration(AmsModule module, void *userData,
 #if RAMSDEBUG
 PUTS("in HandleUnregistration");
 #endif
-	sourceModule = LookupModule(ams_get_unit_nbr(module),
-			ams_get_module_nbr(module), gWay);
+	unitNbr = ams_get_unit_nbr(module);
+	moduleNbr = ams_get_module_nbr(module);
+	if (unitNbr < 0 || moduleNbr < 0)
+	{
+		return;
+	}
+
+	sourceModule = LookupModule(unitNbr, moduleNbr, gWay);
 	if ((elt = ModuleSetMember(sourceModule, gWay->registerSet)) != NULL)
 	{
 		lyst_delete(elt);
