@@ -992,7 +992,7 @@ static int	SendRPDUviaBp(RamsGateway *gWay, RamsNode *ramsNode,
 
 	classOfService = flowLabel & 0x03;
 	ecos.flags = (flowLabel >> 2) & 0x03;
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	extent = sdr_insert(sdr, envelope, envelopeLength);
 	if (extent == 0)
 	{
@@ -1002,15 +1002,16 @@ static int	SendRPDUviaBp(RamsGateway *gWay, RamsNode *ramsNode,
 	}
 
 	bundleZco = zco_create(sdr, ZcoSdrSource, extent, 0, envelopeLength);
-	if (sdr_end_xn(sdr) < 0 || bundleZco == 0)
+	if (sdr_end_xn(sdr) < 0 || bundleZco == (Object) ERROR
+	|| bundleZco == 0)
 	{
 		ErrMsg("Failed creating message.");
 		return -1;
 	}
 
-	if (bp_send(gWay->sap, BP_BLOCKING, ramsNode->gwEid, "dtn:none",
-			gWay->ttl, classOfService, SourceCustodyRequired,
-			0, 0, &ecos, bundleZco, &newBundle) < 1)
+	if (bp_send(gWay->sap, ramsNode->gwEid, "dtn:none", gWay->ttl,
+			classOfService, SourceCustodyRequired, 0, 0, &ecos,
+			bundleZco, &newBundle) < 1)
 	{
 		isprintf(errorMsg, sizeof errorMsg,
 				"Cannot send message to %s.", ramsNode->gwEid);
