@@ -12,13 +12,16 @@
 #include "rfx.h"
 #include "ltp.h"
 #include "bp.h"
+
+#ifndef NASA_PROTECTED_FLIGHT_CODE
 #include "cfdp.h"
+#endif
 
 #define	ION_NODE_NBR	19
 
 static void	createIonConfigFiles()
 {
-	int	nodenbr = ION_NODE_NBR;
+	uvast	nodenbr = ION_NODE_NBR;
 	char	filenamebuf[80];
 	int	fd;
 	char	*ionconfigLines[] =	{
@@ -78,8 +81,8 @@ static void	createIonConfigFiles()
 
 	/*	Create ionconfig file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.ionconfig",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".ionconfig", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -96,7 +99,8 @@ static void	createIonConfigFiles()
 
 	/*	Create ionrc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.ionrc", nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".ionrc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -104,15 +108,14 @@ static void	createIonConfigFiles()
 		return;
 	}
 
-	isprintf(linebuf, sizeof linebuf, "1 %d /ion/node%d.ionconfig\ns\n",
-			nodenbr, nodenbr);
+	isprintf(linebuf, sizeof linebuf, "1 " UVAST_FIELDSPEC " /ion/node"
+			UVAST_FIELDSPEC ".ionconfig\ns\n", nodenbr, nodenbr);
 	oK(iputs(fd, linebuf));
 	close(fd);
 
 	/*	Create global.ionrc file.				*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/global.ionrc",
-			nodenbr);
+	istrcpy(filenamebuf, "/ion/global.ionrc", sizeof filenamebuf);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -129,8 +132,8 @@ static void	createIonConfigFiles()
 
 	/*	Create ionsecrc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.ionsecrc",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".ionsecrc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -147,8 +150,8 @@ static void	createIonConfigFiles()
 
 	/*	Create ltprc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.ltprc",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".ltprc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -165,8 +168,8 @@ static void	createIonConfigFiles()
 
 	/*	Create ipnrc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.ipnrc",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".ipnrc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -183,8 +186,8 @@ static void	createIonConfigFiles()
 
 	/*	Create bprc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.bprc",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".bprc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -197,15 +200,16 @@ static void	createIonConfigFiles()
 		oK(iputs(fd, *line));
 	}
 
-	isprintf(linebuf, sizeof linebuf, "r 'ipnadmin /ion/node%d.ipnrc'\ns\n",
-			nodenbr);
+	isprintf(linebuf, sizeof linebuf, "r 'ipnadmin /ion/node"
+			UVAST_FIELDSPEC ".ipnrc'\ns\n", nodenbr);
 	oK(iputs(fd, linebuf));
 	close(fd);
 
+#ifndef NASA_PROTECTED_FLIGHT_CODE
 	/*	Create cfdprc file.					*/
 
-	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node%d.cfdprc",
-			nodenbr);
+	isprintf(filenamebuf, sizeof filenamebuf, "/ion/node" UVAST_FIELDSPEC
+			".cfdprc", nodenbr);
 	fd = iopen(filenamebuf, O_WRONLY | O_CREAT | O_TRUNC, 0777);
 	if (fd < 0)
 	{
@@ -215,16 +219,18 @@ static void	createIonConfigFiles()
 
 	oK(iputs(fd, "1\ns bputa\n"));
 	close(fd);
+#endif
 }
 
 static int	startDTN()
 {
-	int	nodenbr = ION_NODE_NBR;
+	uvast	nodenbr = ION_NODE_NBR;
 	char	cmd[80];
 	int	count;
 
 	sm_ipc_init();
-	isprintf(cmd, sizeof cmd, "ionadmin /ion/node%d.ionrc", nodenbr);
+	isprintf(cmd, sizeof cmd, "ionadmin /ion/node" UVAST_FIELDSPEC
+			".ionrc", nodenbr);
 	pseudoshell(cmd);
 	count = 5;
 	while (rfx_system_is_started() == 0)
@@ -240,13 +246,15 @@ static int	startDTN()
 
 	pseudoshell("ionadmin /ion/global.ionrc");
 	snooze(1);
-	isprintf(cmd, sizeof cmd, "ionsecadmin /ion/node%d.ionsecrc", nodenbr);
+	isprintf(cmd, sizeof cmd, "ionsecadmin /ion/node" UVAST_FIELDSPEC
+			".ionsecrc", nodenbr);
 	pseudoshell(cmd);
 	snooze(1);
 
 	/*	Now start the higher layers of the DTN stack.		*/
 
-	isprintf(cmd, sizeof cmd, "ltpadmin /ion/node%d.ltprc", nodenbr);
+	isprintf(cmd, sizeof cmd, "ltpadmin /ion/node" UVAST_FIELDSPEC
+			".ltprc", nodenbr);
 	pseudoshell(cmd);
 	count = 5;
 	while (ltp_engine_is_started() == 0)
@@ -260,7 +268,8 @@ static int	startDTN()
 		}
 	}
 
-	isprintf(cmd, sizeof cmd, "bpadmin /ion/node%d.bprc", nodenbr);
+	isprintf(cmd, sizeof cmd, "bpadmin /ion/node" UVAST_FIELDSPEC
+			".bprc", nodenbr);
 	pseudoshell(cmd);
 	count = 5;
 	while (bp_agent_is_started() == 0)
@@ -274,13 +283,16 @@ static int	startDTN()
 		}
 	}
 
-	isprintf(cmd, sizeof cmd, "lgagent ipn:%d.127", nodenbr);
+	isprintf(cmd, sizeof cmd, "lgagent ipn:" UVAST_FIELDSPEC ".127",
+			nodenbr);
 	pseudoshell(cmd);
 	snooze(1);
 
+#ifndef NASA_PROTECTED_FLIGHT_CODE
 	/*	Now start CFDP.						*/
 
-	isprintf(cmd, sizeof cmd, "cfdpadmin /ion/node%d.cfdprc", nodenbr);
+	isprintf(cmd, sizeof cmd, "cfdpadmin /ion/node" UVAST_FIELDSPEC
+			".cfdprc", nodenbr);
 	pseudoshell(cmd);
 	count = 5;
 	while (cfdp_entity_is_started() == 0)
@@ -293,7 +305,7 @@ static int	startDTN()
 			return -1;
 		}
 	}
-
+#endif
 	return 0;
 }
 
@@ -302,19 +314,21 @@ static void	testLoopback()
 	char	cmd[80];
 
 	puts("Starting loopback test.");
-	isprintf(cmd, sizeof cmd, "bpsink ipn:%d.1", ION_NODE_NBR);
+	isprintf(cmd, sizeof cmd, "bpsink ipn:" UVAST_FIELDSPEC ".1",
+			ION_NODE_NBR);
 	pseudoshell(cmd);
 	snooze(1);
-	isprintf(cmd, sizeof cmd, "bpsource ipn:%d.1 'Hello, world.'",
-			ION_NODE_NBR);
+	isprintf(cmd, sizeof cmd, "bpsource ipn:" UVAST_FIELDSPEC
+			".1 'Hello, world.'", ION_NODE_NBR);
 	pseudoshell(cmd);
 	snooze(1);
 	puts("Loopback test ended.");
 }
 
-int	stopDTN(int a1, int a2, int a3, int a4, int a5,
-		int a6, int a7, int a8, int a9, int a10)
+static int	stopDTN(int a1, int a2, int a3, int a4, int a5,
+			int a6, int a7, int a8, int a9, int a10)
 {
+#ifndef NASA_PROTECTED_FLIGHT_CODE
 	/*	Stop CFDP.						*/
 
 	pseudoshell("cfdpadmin .");
@@ -322,6 +336,7 @@ int	stopDTN(int a1, int a2, int a3, int a4, int a5,
 	{
 		snooze(1);
 	}
+#endif
 
 	/*	Stop BP.						*/
 
@@ -364,6 +379,7 @@ rtems_task	Init(rtems_task_argument ignored)
 	}
 
 	testLoopback();
+	snooze(1);
 	puts("Stopping ION.");
 	oK(stopDTN(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 	puts("ION stopped.");
