@@ -45,12 +45,31 @@ static BssDB	*_bssConstants()
 {
 	static BssDB	buf;
 	static BssDB	*db = NULL;
+	Sdr		sdr;
+	Object		dbObject;
 
 	if (db == NULL)
 	{
-		sdr_read(getIonsdr(), (char *) &buf, _bssdbObject(NULL),
-				sizeof(BssDB));
-		db = &buf;
+		sdr = getIonsdr();
+		CHKNULL(sdr);
+		dbObject = _bssdbObject(NULL);
+		if (dbObject)
+		{
+			if (sdr_heap_is_halted(sdr))
+			{
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(BssDB));
+			}
+			else
+			{
+				CHKNULL(sdr_begin_xn(sdr));
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(BssDB));
+				sdr_exit_xn(sdr);
+			}
+
+			db = &buf;
+		}
 	}
 
 	return db;
@@ -84,7 +103,7 @@ static int	lookupBssEid(char *uriBuffer, char *neighborClId)
 			 *	neighbor's EID.				*/
 
 			isprintf(uriBuffer, SDRSTRING_BUFSZ,
-					"ipn:%lu.0", plan->nodeNbr);
+				"ipn:" UVAST_FIELDSPEC ".0", plan->nodeNbr);
 			return 1;
 		}
 	}
@@ -109,7 +128,7 @@ int	ipnInit()
 
 	/*	Recover the BSS database, creating it if necessary.	*/
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	oK(senderEidLookupFunctions(lookupBssEid));
 	bssdbObject = sdr_find(sdr, BSS_DBNAME, NULL);
 	switch (bssdbObject)
@@ -167,8 +186,8 @@ Object locateBssEntry (CbheEid dst, Object *nextEntry)
 	Object	elt;
 		OBJ_POINTER(bssEntry, entry);
 
-	unsigned long serviceNbr = dst.serviceNbr; 
-	unsigned long nodeNbr = dst.nodeNbr;
+	unsigned int serviceNbr = dst.serviceNbr; 
+	uvast nodeNbr = dst.nodeNbr;
 
 	/*  
 	 *  This function locates the bssEntry identified by the
@@ -192,7 +211,7 @@ Object locateBssEntry (CbheEid dst, Object *nextEntry)
 
 		if (entry->dstServiceNbr > serviceNbr)
 		{
-			if (entry->dstServiceNbr != BSS_ALL_OTHERS)
+			if (entry->dstServiceNbr != BSS_ALL_OTHER_SERVICES)
 			{						
 				if (nextEntry) *nextEntry = elt;
 				break;	/*	Same as end of list.	*/
@@ -208,7 +227,7 @@ Object locateBssEntry (CbheEid dst, Object *nextEntry)
 
 		if (entry->dstNodeNbr > nodeNbr)
 		{
-			if (entry->dstNodeNbr != BSS_ALL_OTHERS)
+			if (entry->dstNodeNbr != BSS_ALL_OTHER_NODES)
 			{					
 				if (nextEntry) *nextEntry = elt;
 				break;	/*	Same as end of list.	*/
@@ -222,13 +241,13 @@ Object locateBssEntry (CbheEid dst, Object *nextEntry)
 	return 0;
 }
 
-int bss_addBssEntry(long argServiceNbr, long argNodeNbr)
+int bss_addBssEntry(int argServiceNbr, vast argNodeNbr)
 
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	dstServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	dstNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	dstServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		dstNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	
 	bssEntry	entry;
@@ -238,15 +257,16 @@ int bss_addBssEntry(long argServiceNbr, long argNodeNbr)
 	char 		memo[256];
 
 	CHKERR(argServiceNbr && argNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	eid.serviceNbr = dstServiceNbr;
 	eid.nodeNbr = dstNodeNbr;
 
 	if (locateBssEntry(eid, &nextEntry) !=0)
 	{
 		sdr_exit_xn(sdr);
-		isprintf(memo, sizeof memo, "[?] BSS duplicate entry: %lu-%lu", 
-				 dstServiceNbr,dstNodeNbr);
+		isprintf(memo, sizeof memo,
+				"[?] BSS duplicate entry: %u-" UVAST_FIELDSPEC, 
+				dstServiceNbr,dstNodeNbr);
 		writeMemo(memo);
 		return 0;
 	}
@@ -279,12 +299,12 @@ int bss_addBssEntry(long argServiceNbr, long argNodeNbr)
 	return 1;
 }
 
-int bss_removeBssEntry(long argServiceNbr, long argNodeNbr)
+int bss_removeBssEntry(int argServiceNbr, vast argNodeNbr)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	dstServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	dstNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	dstServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		dstNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object	elt;
 	Object	entryObj;
@@ -293,7 +313,7 @@ int bss_removeBssEntry(long argServiceNbr, long argNodeNbr)
 	eid.nodeNbr = dstNodeNbr;
 
 	CHKERR(argServiceNbr && argNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateBssEntry(eid, NULL);
 	if (elt == 0)
 	{
@@ -323,10 +343,10 @@ static LystElt locateStream (Lyst loggedStreams, Bundle *bundle,
 {	
 	LystElt elt;
 	stream *strm = NULL;
-	unsigned long srcNodeNbr = bundle->id.source.c.nodeNbr;
-	unsigned long srcServiceNbr = bundle->id.source.c.serviceNbr;
-	unsigned long dstNodeNbr = bundle->destination.c.nodeNbr;
-	unsigned long dstServiceNbr = bundle->destination.c.serviceNbr;
+	uvast srcNodeNbr = bundle->id.source.c.nodeNbr;
+	unsigned int srcServiceNbr = bundle->id.source.c.serviceNbr;
+	uvast dstNodeNbr = bundle->destination.c.nodeNbr;
+	unsigned int dstServiceNbr = bundle->destination.c.serviceNbr;
 
 	/*	
 	 *  This function locates the BSS stream identified by the 
@@ -461,7 +481,7 @@ void bss_monitorStream(Lyst loggedStreams, Bundle bundle)
 	}
 }
 
-static Object	locatePlan(unsigned long nodeNbr, Object *nextPlan)
+static Object	locatePlan(uvast nodeNbr, Object *nextPlan)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -492,7 +512,7 @@ static Object	locatePlan(unsigned long nodeNbr, Object *nextPlan)
 	return 0;
 }
 
-void	bss_findPlan(unsigned long nodeNbr, Object *planAddr, Object *eltp)
+void	bss_findPlan(uvast nodeNbr, Object *planAddr, Object *eltp)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -529,9 +549,9 @@ static void	createXmitDirective(FwdDirective *directive,
 	}
 }
 
-int	bss_addPlan(unsigned long nodeNbr, DuctExpression *defaultDuct,
+int	bss_addPlan(uvast nodeNbr, DuctExpression *defaultDuct,
 		    DuctExpression *rtDuct, DuctExpression *pbDuct,
-		    unsigned long expectedRTT)
+		    unsigned int expectedRTT)
 {
 	Sdr	sdr = getIonsdr();
 	Object	nextPlan;
@@ -539,7 +559,7 @@ int	bss_addPlan(unsigned long nodeNbr, DuctExpression *defaultDuct,
 	Object	planObj;
 
 	CHKERR(nodeNbr && defaultDuct);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (locatePlan(nodeNbr, &nextPlan) != 0)
 	{
 		sdr_exit_xn(sdr);
@@ -588,7 +608,7 @@ static void	destroyXmitDirective(FwdDirective *directive)
 	}
 }
 
-int	bss_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct,
+int	bss_updatePlan(uvast nodeNbr, DuctExpression *defaultDuct,
 		    DuctExpression *rtDuct, DuctExpression *pbDuct,
 		    int expectedRTT)
 {
@@ -598,7 +618,7 @@ int	bss_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct,
 	BssPlan	plan;
 
 	CHKERR(nodeNbr && defaultDuct);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -629,7 +649,7 @@ int	bss_updatePlan(unsigned long nodeNbr, DuctExpression *defaultDuct,
 	return 1;
 }
 
-int	bss_removePlan(unsigned long nodeNbr)
+int	bss_removePlan(uvast nodeNbr)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -637,7 +657,7 @@ int	bss_removePlan(unsigned long nodeNbr)
 		OBJ_POINTER(BssPlan, plan);
 
 	CHKERR(nodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -673,8 +693,8 @@ int	bss_removePlan(unsigned long nodeNbr)
 	return 1;
 }
 
-static Object	locateRule(Object rules, unsigned long srcServiceNbr,
-			unsigned long srcNodeNbr, Object *nextRule)
+static Object	locateRule(Object rules, unsigned int srcServiceNbr,
+			uvast srcNodeNbr, Object *nextRule)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -723,13 +743,13 @@ static Object	locateRule(Object rules, unsigned long srcServiceNbr,
 	return 0;
 }
 
-void	bss_findPlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, BssPlan *plan, Object *ruleAddr, Object *eltp)
+void	bss_findPlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr,
+		BssPlan *plan, Object *ruleAddr, Object *eltp)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(BssPlan, planPtr);
@@ -769,14 +789,14 @@ void	bss_findPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	*eltp = elt;
 }
 
-int	bss_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, DuctExpression *directive, 
-		DuctExpression *rtDuct, DuctExpression *pbDuct)
+int	bss_addPlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr,
+		DuctExpression *directive, DuctExpression *rtDuct,
+		DuctExpression *pbDuct)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(BssPlan, plan);
@@ -785,7 +805,7 @@ int	bss_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	Object		addr;
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -838,14 +858,14 @@ int	bss_addPlanRule(unsigned long nodeNbr, long argServiceNbr,
 	return 1;
 }
 
-int	bss_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr, DuctExpression *directive, 
-		DuctExpression *rtDuct, DuctExpression *pbDuct)
+int	bss_updatePlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr,
+		DuctExpression *directive, DuctExpression *rtDuct,
+		DuctExpression *pbDuct)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(BssPlan, plan);
@@ -853,7 +873,7 @@ int	bss_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
 	BssRule		ruleBuf;
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -892,13 +912,12 @@ int	bss_updatePlanRule(unsigned long nodeNbr, long argServiceNbr,
 	return 1;
 }
 
-int	bss_removePlanRule(unsigned long nodeNbr, long argServiceNbr,
-		long argNodeNbr)
+int	bss_removePlanRule(uvast nodeNbr, int argServiceNbr, vast argNodeNbr)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(BssPlan, plan);
@@ -906,7 +925,7 @@ int	bss_removePlanRule(unsigned long nodeNbr, long argServiceNbr,
 			OBJ_POINTER(BssRule, rule);
 
 	CHKERR(nodeNbr && srcNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locatePlan(nodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -955,16 +974,17 @@ int bss_copyDirective(Bundle *bundle, FwdDirective *directive,
 	if(locateBssEntry(bundle->destination.c, NULL)!=0)
 	{
 		streamElt = locateStream(loggedStreams, bundle, NULL);
-	
 		if (streamElt == NULL)
 		{
+			#if LIBBSSFWDEBUG
+			writeMemo("No entry detected in loggedStreams");
+			#endif
 			if(rtDirective->outductElt!=0)
 			{						
 				memcpy((char *) directive, (char *) rtDirective,
 					sizeof(FwdDirective));
 				#if LIBBSSFWDEBUG
-				printf("No entry detected in loggedStreams list\n");
-				printf("Real-time mode directive was returned\n");
+				writeMemo("Real-time directive used.");
 				#endif
 			}
 			else
@@ -973,29 +993,33 @@ int bss_copyDirective(Bundle *bundle, FwdDirective *directive,
 					(char *) defaultDirective,
 					sizeof(FwdDirective));
 				#if LIBBSSFWDEBUG
-				printf("No entry detected in loggedStreams list\n");
-				printf("The default directive was returned\n");
+				writeMemo("Default directive used.");
 				#endif
 			}
 		}
 		else
 		{			
+			#if LIBBSSFWDEBUG
+			writeMemo("Found entry in loggedStreams");
+			#endif
 			strm = (stream *) lyst_data(streamElt);
-				
 			if (bundle->id.creationTime.seconds > 
 			    strm->latestTimeLogged.seconds || 
 			    (bundle->id.creationTime.seconds == 
 			    strm->latestTimeLogged.seconds && 
 		    	    bundle->id.creationTime.count > 
 			    strm->latestTimeLogged.count))
-			{											
+			{
+				#if LIBBSSFWDEBUG
+				writeMemo("Record is in sequence.");
+				#endif
 				if(rtDirective->outductElt!=0)
 				{					
 					memcpy((char *) directive, 
 						(char *) rtDirective,
 						sizeof(FwdDirective));
 					#if LIBBSSFWDEBUG
-					printf("Real-time mode directive was returned\n");
+					writeMemo("Real-time directive used.");
 					#endif
 				}
 				else
@@ -1004,19 +1028,22 @@ int bss_copyDirective(Bundle *bundle, FwdDirective *directive,
 						(char *) defaultDirective,
 						sizeof(FwdDirective));
 					#if LIBBSSFWDEBUG
-					printf("The default directive was returned\n");
+					writeMemo("Default directive used.");
 					#endif
 				}	
 			}
 			else
 			{
+				#if LIBBSSFWDEBUG
+				writeMemo("Record out of sequence.");
+				#endif
 				if(pbDirective->outductElt!=0)
 				{						
 					memcpy((char *) directive, 
 						(char *) pbDirective,
 						sizeof(FwdDirective));
 					#if LIBBSSFWDEBUG
-					printf("Playback mode directive was returned\n");
+					writeMemo("Playback directive used.");
 					#endif
 				}
 				else
@@ -1025,7 +1052,7 @@ int bss_copyDirective(Bundle *bundle, FwdDirective *directive,
 						(char *) defaultDirective,
 						sizeof(FwdDirective));
 					#if LIBBSSFWDEBUG
-					printf("The default directive was returned\n");
+					writeMemo("Default directive used.");
 					#endif
 				}
 			}
@@ -1036,16 +1063,16 @@ int bss_copyDirective(Bundle *bundle, FwdDirective *directive,
 		memcpy((char *) directive, (char *) defaultDirective,
 			sizeof(FwdDirective));
 		#if LIBBSSFWDEBUG
-		printf("Not a BSS traffic bundle\n");
-		printf("The default directive was returned\n");
+		writeMemo("Not a BSS traffic bundle.");
+		writeMemo("Default directive used.");
 		#endif
 	}
 
 	return 1;		
 }
 
-static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
-			unsigned long sourceNodeNbr, Bundle *bundle,
+static int	lookupRule(Object rules, unsigned int sourceServiceNbr,
+			uvast sourceNodeNbr, Bundle *bundle,
 			FwdDirective *dirbuf, Lyst loggedStreams)
 {
 	Sdr	sdr = getIonsdr();
@@ -1053,7 +1080,7 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 	Object	elt;
 		OBJ_POINTER(BssRule, rule);
 
-	/*	Universal wild-card match (BSS_ALL_OTHERS), if any,
+	/*	Universal wild-card match (BSS_ALL_OTHER_xxx), if any,
 	 *	is at the end of the list, so there's no way to
 	 *	terminate the search early.				*/
 
@@ -1068,7 +1095,7 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 		}
 
 		if (rule->srcServiceNbr > sourceServiceNbr
-			&& rule->srcServiceNbr != BSS_ALL_OTHERS)
+			&& rule->srcServiceNbr != BSS_ALL_OTHER_SERVICES)
 		{
 			continue;
 		}
@@ -1081,7 +1108,7 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 		}
 
 		if (rule->srcNodeNbr > sourceNodeNbr
-			&& rule->srcNodeNbr != BSS_ALL_OTHERS)
+			&& rule->srcNodeNbr != BSS_ALL_OTHER_NODES)
 		{
 			continue;
 		}
@@ -1099,9 +1126,9 @@ static int	lookupRule(Object rules, unsigned long sourceServiceNbr,
 	return 0;
 }
 
-int	bss_lookupPlanDirective(unsigned long nodeNbr,
-		unsigned long sourceServiceNbr, unsigned long sourceNodeNbr,
-		Bundle *bundle, FwdDirective *dirbuf, Lyst loggedStreams)
+int	bss_lookupPlanDirective(uvast nodeNbr, unsigned int sourceServiceNbr,
+		uvast sourceNodeNbr, Bundle *bundle, FwdDirective *dirbuf,
+		Lyst loggedStreams)
 {
 	Sdr	sdr = getIonsdr();
 	Object	addr;
@@ -1136,7 +1163,7 @@ int	bss_lookupPlanDirective(unsigned long nodeNbr,
 	return 1;
 }
 
-static int getRTT(unsigned long pxNodeNbr)
+static int getRTT(uvast pxNodeNbr)
 {
 	Sdr	sdr = getIonsdr();
 	Object	addr;
@@ -1210,8 +1237,8 @@ int 	bss_setCtDueTimer(Bundle bundle, Object bundleAddr)
 	return -1;
 }
 
-static Object	locateGroup(unsigned long firstNodeNbr,
-			unsigned long lastNodeNbr, Object *nextGroup)
+static Object	locateGroup(uvast firstNodeNbr, uvast lastNodeNbr,
+			Object *nextGroup)
 {
 	Sdr	sdr = getIonsdr();
 	int	targetSize;
@@ -1261,8 +1288,8 @@ static Object	locateGroup(unsigned long firstNodeNbr,
 	return 0;
 }
 
-void	bss_findGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		Object *groupAddr, Object *eltp)
+void	bss_findGroup(uvast firstNodeNbr, uvast lastNodeNbr, Object *groupAddr,
+		Object *eltp)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -1284,8 +1311,7 @@ void	bss_findGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	*eltp = elt;
 }
 
-int	bss_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		char *viaEid)
+int	bss_addGroup(uvast firstNodeNbr, uvast lastNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
 	Object		nextGroup;
@@ -1295,7 +1321,7 @@ int	bss_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	CHKERR(firstNodeNbr && lastNodeNbr && viaEid);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
 	CHKERR(strlen(viaEid) <= MAX_SDRSTRING);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (locateGroup(firstNodeNbr, lastNodeNbr, &nextGroup) != 0)
 	{
 		sdr_exit_xn(sdr);
@@ -1336,8 +1362,7 @@ int	bss_addGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	bss_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		char *viaEid)
+int	bss_updateGroup(uvast firstNodeNbr, uvast lastNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
 	Object		elt;
@@ -1347,7 +1372,7 @@ int	bss_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	CHKERR(firstNodeNbr && lastNodeNbr && viaEid);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
 	CHKERR(strlen(viaEid) <= MAX_SDRSTRING);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1372,7 +1397,7 @@ int	bss_updateGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	bss_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
+int	bss_removeGroup(uvast firstNodeNbr, uvast lastNodeNbr)
 {
 	Sdr	sdr = getIonsdr();
 	Object	elt;
@@ -1381,7 +1406,7 @@ int	bss_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
 
 	CHKERR(firstNodeNbr && lastNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1414,14 +1439,14 @@ int	bss_removeGroup(unsigned long firstNodeNbr, unsigned long lastNodeNbr)
 	return 1;
 }
 
-void	bss_findGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		long argServiceNbr, long argNodeNbr, IpnGroup *group,
+void	bss_findGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, IpnGroup *group,
 		Object *ruleAddr, Object *eltp)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, groupPtr);
@@ -1462,13 +1487,13 @@ void	bss_findGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	*eltp = elt;
 }
 
-int	bss_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
-		long argServiceNbr, long argNodeNbr, char *viaEid)
+int	bss_addGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -1478,7 +1503,7 @@ int	bss_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1525,14 +1550,13 @@ int	bss_addGroupRule(unsigned long firstNodeNbr, unsigned long lastNodeNbr,
 	return 1;
 }
 
-int	bss_updateGroupRule(unsigned long firstNodeNbr,
-		unsigned long lastNodeNbr, long argServiceNbr, long argNodeNbr,
-		char *viaEid)
+int	bss_updateGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr, char *viaEid)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -1541,7 +1565,7 @@ int	bss_updateGroupRule(unsigned long firstNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1575,13 +1599,13 @@ int	bss_updateGroupRule(unsigned long firstNodeNbr,
 	return 1;
 }
 
-int	bss_removeGroupRule(unsigned long firstNodeNbr,
-		unsigned long lastNodeNbr, long argServiceNbr, long argNodeNbr)
+int	bss_removeGroupRule(uvast firstNodeNbr, uvast lastNodeNbr,
+		int argServiceNbr, vast argNodeNbr)
 {
 	Sdr		sdr = getIonsdr();
-	unsigned long	srcServiceNbr = (argServiceNbr == -1 ? BSS_ALL_OTHERS
-				: argServiceNbr);
-	unsigned long	srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHERS
+	unsigned int	srcServiceNbr = (argServiceNbr == -1 ?
+				BSS_ALL_OTHER_SERVICES : argServiceNbr);
+	uvast		srcNodeNbr = (argNodeNbr == -1 ? BSS_ALL_OTHER_NODES
 				: argNodeNbr);
 	Object		elt;
 			OBJ_POINTER(IpnGroup, group);
@@ -1590,7 +1614,7 @@ int	bss_removeGroupRule(unsigned long firstNodeNbr,
 
 	CHKERR(firstNodeNbr && lastNodeNbr && srcNodeNbr);
 	CHKERR(firstNodeNbr <= lastNodeNbr);
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateGroup(firstNodeNbr, lastNodeNbr, NULL);
 	if (elt == 0)
 	{
@@ -1624,9 +1648,9 @@ int	bss_removeGroupRule(unsigned long firstNodeNbr,
 	return 1;
 }
 
-int	bss_lookupGroupDirective(unsigned long nodeNbr,
-		unsigned long sourceServiceNbr, unsigned long sourceNodeNbr,
-		Bundle *bundle, FwdDirective *dirbuf, Lyst loggedStreams)
+int	bss_lookupGroupDirective(uvast nodeNbr, unsigned int sourceServiceNbr,
+		uvast sourceNodeNbr, Bundle *bundle, FwdDirective *dirbuf,
+		Lyst loggedStreams)
 {
 	Sdr		sdr = getIonsdr();
 	Object		elt;
