@@ -9,14 +9,13 @@
 	
 									*/
 #include "bpP.h"
+#include "sdrhash.h"
 #include "smlist.h"
 
 #ifdef ENABLE_BPACS
-#include "acs/acs.h"		/* provides sendAcs */
+#include "acs.h"		/* provides sendAcs */
 #endif /* ENABLE_ACS */
 
-extern void	manageProductionThrottle(BpVdb *vdb);
-
 static long	_running(long *newValue)
 {
 	void	*value;
@@ -51,8 +50,7 @@ static int	dispatchEvents(Sdr sdr, Object events, time_t currentTime)
 
 	while (1)
 	{
-		sdr_begin_xn(sdr);
-		CHKERR(ionLocked());	/*	In case of killm.	*/
+		CHKERR(sdr_begin_xn(sdr));
 		elt = sdr_list_first(sdr, events);
 		if (elt == 0)	/*	No more events to dispatch.	*/
 		{
@@ -137,7 +135,7 @@ static int	adjustThrottles()
 	BpVdb		*bpvdb = getBpVdb();
 	PsmAddress	elt;
 	VOutduct	*outduct;
-	unsigned long	nodeNbr;
+	uvast		nodeNbr;
 	IonNeighbor	*neighbor;
 	PsmAddress	nextElt;
 	int		mustPrintStats = 0;
@@ -184,7 +182,7 @@ static int	adjustThrottles()
 			continue;
 		}
 
-		nodeNbr = atol(outduct->ductName);
+		nodeNbr = strtouvast(outduct->ductName);
 		neighbor = findNeighbor(ionvdb, nodeNbr, &nextElt);
 		if (neighbor == NULL)
 		{
@@ -277,18 +275,14 @@ static int	adjustThrottles()
 
 static void	applyRateControl(Sdr sdr)
 {
-	BpVdb		*vdb = getBpVdb();
 	PsmPartition	ionwm = getIonwm();
+	BpVdb		*vdb = getBpVdb();
 	Throttle	*throttle;
 	PsmAddress	elt;
 	VInduct		*induct;
 	VOutduct	*outduct;
 
-	sdr_begin_xn(sdr);	/*	Just to lock memory.		*/
-
-	/*	Recalculate limit on local bundle generation.		*/
-
-	manageProductionThrottle(vdb);
+	CHKVOID(sdr_begin_xn(sdr));
 
 	/*	Enable some bundle acquisition.				*/
 
@@ -302,7 +296,7 @@ static void	applyRateControl(Sdr sdr)
 			continue;	/*	Not rate-controlled.	*/
 		}
 
-		if (throttle->capacity <= 0)
+		if (throttle->capacity <= throttle->nominalRate)
 		{
 			throttle->capacity += throttle->nominalRate;
 		}
@@ -325,7 +319,7 @@ static void	applyRateControl(Sdr sdr)
 			continue;	/*	Not rate-controlled.	*/
 		}
 
-		if (throttle->capacity <= 0)
+		if (throttle->capacity <= throttle->nominalRate)
 		{
 			throttle->capacity += throttle->nominalRate;
 		}
@@ -336,7 +330,7 @@ static void	applyRateControl(Sdr sdr)
 		}
 	}
 
-	sdr_exit_xn(sdr);	/*	Unlock memory.			*/
+	oK(sdr_end_xn(sdr));
 }
 
 #if defined (VXWORKS) || defined (RTEMS) || defined (bionic)
