@@ -67,8 +67,10 @@ static void	*sendBundles(void *parm)
 	}
 
 	sdr = getIonsdr();
+	CHKNULL(sdr_begin_xn(sdr));
 	sdr_read(sdr, (char *) &outduct, sdr_list_data(sdr,
 			parms->vduct->outductElt), sizeof(Outduct));
+	sdr_exit_xn(sdr);
 	memset((char *) outflows, 0, sizeof outflows);
 	outflows[0].outboundBundles = outduct.bulkQueue;
 	outflows[1].outboundBundles = outduct.stdQueue;
@@ -88,7 +90,7 @@ static void	*sendBundles(void *parm)
 		}
 
 		if (bpDequeue(parms->vduct, outflows, &bundleZco,
-			&extendedCOS, destDuctName, DGRCLA_BUFSZ, -1) < 0)
+				&extendedCOS, destDuctName, 64000, -1) < 0)
 		{
 			threadRunning = 0;
 			writeMemo("[?] dgrcla failed de-queueing bundle.");
@@ -106,7 +108,7 @@ static void	*sendBundles(void *parm)
 			portNbr = DGRCLA_PORT_NBR;
 		}
 
-		sdr_begin_xn(sdr);
+		CHKNULL(sdr_begin_xn(sdr));
 		if (hostNbr == 0)		/*	Can't send it.	*/
 		{
 			failedTransmissions++;
@@ -156,7 +158,7 @@ failure.", NULL);
 			}
 		}
 
-		sdr_begin_xn(sdr);
+		CHKNULL(sdr_begin_xn(sdr));
 		zco_destroy(sdr, bundleZco);
 		if (sdr_end_xn(sdr) < 0)
 		{
@@ -247,11 +249,12 @@ static void	*receiveBundles(void *parm)
 			switch (rc)
 			{
 				case DgrDatagramAcknowledged:
-					sdr_begin_xn(sdr);
+					CHKNULL(sdr_begin_xn(sdr));
 					bundleZco = zco_create(sdr,
 						ZcoSdrSource, sdr_insert(sdr,
 						buffer, length), 0, length);
-					if (sdr_end_xn(sdr) < 0)
+					if (sdr_end_xn(sdr) < 0
+					|| bundleZco == (Object) ERROR)
 					{
 						putErrmsg("Failed creating \
 temporary ZCO.", NULL);
@@ -259,6 +262,15 @@ temporary ZCO.", NULL);
 						break;	/*	Switch.	*/
 					}
 
+					if (bundleZco == 0)
+					{
+						/*	No ZCO space;
+						 *	in effect,
+						 *	datagram lost.	*/
+
+						continue;
+					}
+
 					if (bpHandleXmitSuccess(bundleZco, 0))
 					{
 						threadRunning = 0;
@@ -266,7 +278,7 @@ temporary ZCO.", NULL);
 success.", NULL);
 					}
 
-					sdr_begin_xn(sdr);
+					CHKNULL(sdr_begin_xn(sdr));
 					zco_destroy(sdr, bundleZco);
 					if (sdr_end_xn(sdr) < 0)
 					{
@@ -283,11 +295,12 @@ bundle ZCO.", NULL);
 					continue;
 
 				case DgrDatagramNotAcknowledged:
-					sdr_begin_xn(sdr);
+					CHKNULL(sdr_begin_xn(sdr));
 					bundleZco = zco_create(sdr,
 						ZcoSdrSource, sdr_insert(sdr,
 						buffer, length), 0, length);
-					if (sdr_end_xn(sdr) < 0)
+					if (sdr_end_xn(sdr) < 0
+					|| bundleZco == (Object) ERROR)
 					{
 						putErrmsg("Failed creating \
 temporary ZCO.", NULL);
@@ -295,6 +308,15 @@ temporary ZCO.", NULL);
 						break;	/*	Switch.	*/
 					}
 
+					if (bundleZco == 0)
+					{
+						/*	No ZCO space;
+						 *	in effect,
+						 *	datagram lost.	*/
+
+						continue;
+					}
+
 					if (bpHandleXmitFailure(bundleZco))
 					{
 						threadRunning = 0;
@@ -302,7 +324,7 @@ temporary ZCO.", NULL);
 failure.", NULL);
 					}
 
-					sdr_begin_xn(sdr);
+					CHKNULL(sdr_begin_xn(sdr));
 					zco_destroy(sdr, bundleZco);
 					if (sdr_end_xn(sdr) < 0)
 					{
@@ -352,16 +374,9 @@ bundle ZCO.", NULL);
 
 		/*	Must have received a datagram.			*/
 
-		if (getInternetHostName(fromHostNbr, hostName))
-		{
-			senderEid = senderEidBuffer;
-			getSenderEid(&senderEid, hostName);
-		}
-		else
-		{
-			senderEid = NULL;
-		}
-
+		printDottedString(fromHostNbr, hostName);
+		senderEid = senderEidBuffer;
+		getSenderEid(&senderEid, hostName);
 		if (bpBeginAcq(work, 0, senderEid) < 0
 		|| bpContinueAcq(work, buffer, length) < 0
 		|| bpEndAcq(work) < 0)
@@ -443,7 +458,7 @@ int	main(int argc, char *argv[])
 		return 1;
 	}
 
-	findOutduct("dgr", ductName, &voutduct, &vductElt);
+	findOutduct("dgr", "*", &voutduct, &vductElt);
 	if (vductElt == 0)
 	{
 		putErrmsg("No such dgr outduct.", ductName);
@@ -453,9 +468,11 @@ int	main(int argc, char *argv[])
 	/*	All command-line arguments are now validated.		*/
 
 	sdr = getIonsdr();
+	CHKZERO(sdr_begin_xn(sdr));
 	sdr_read(sdr, (char *) &induct, sdr_list_data(sdr, vinduct->inductElt),
 			sizeof(Induct));
 	sdr_read(sdr, (char *) &protocol, induct.protocol, sizeof(ClProtocol));
+	sdr_exit_xn(sdr);
 	if (protocol.nominalRate == 0)
 	{
 		vinduct->acqThrottle.nominalRate = DEFAULT_DGR_RATE;
@@ -501,7 +518,7 @@ int	main(int argc, char *argv[])
 	senderParms.vduct = voutduct;
 	senderParms.running = &running;
 	senderParms.dgrSap = dgrSap;
-	if (pthread_create(&senderThread, NULL, sendBundles, &senderParms))
+	if (pthread_begin(&senderThread, NULL, sendBundles, &senderParms))
 	{
 		dgr_close(dgrSap);
 		putSysErrmsg("dgrcla can't create sender thread", NULL);
@@ -513,7 +530,7 @@ int	main(int argc, char *argv[])
 	rtp.vduct = vinduct;
 	rtp.running = &running;
 	rtp.dgrSap = dgrSap;
-	if (pthread_create(&receiverThread, NULL, receiveBundles, &rtp))
+	if (pthread_begin(&receiverThread, NULL, receiveBundles, &rtp))
 	{
 		sm_SemEnd(voutduct->semaphore);
 		pthread_join(senderThread, NULL);
