@@ -12,6 +12,12 @@
 
 #include "crypto.h"
 
+/*****************************************************************************
+ *                           CONSTANTS DEFINITIONS                           *
+ *****************************************************************************/
+char* crypto_suite_name="NULL_SUITES";
+
+
 /*****************************************************************************
  *                     HMAC-SHA-1 FUNCTION DEFINITIONS                        *
  *****************************************************************************/
