@@ -13,13 +13,14 @@
 
 /*	Add external function declarations between here...		*/
 
-#include "ecos/ecos.h"
-#include "bsp/extbspbab.h"
-#include "bsp/extbsppcb.h"
-#include "bsp/extbsppib.h"
+#include "ecos.h"
+#include "bae.h"
+#include "extbspbab.h"
+#include "extbsppcb.h"
+#include "extbsppib.h"
 
 #ifdef ENABLE_BPACS
-#include "cteb/cteb.h"
+#include "cteb.h"
 #endif /* ENABLE_BPACS */
 
 /*	... and here.							*/
@@ -55,6 +56,20 @@ static ExtensionDef	extensions[] =
 				ecos_processOnDequeue,
 				0}
 		},
+		{ "bae", EXTENSION_TYPE_BAE, 0,
+				bae_offer,
+				bae_release,
+				bae_acquire,
+				bae_check,
+				bae_record,
+				bae_clear,
+				bae_copy,
+				{bae_processOnFwd,
+				bae_processOnAccept,
+				bae_processOnEnqueue,
+				bae_processOnDequeue,
+				0}
+		},
 #ifdef ENABLE_BPACS
         	{ "cteb", EXTENSION_TYPE_CTEB, 0,
 				cteb_offer,
