@@ -8,7 +8,11 @@
 #include "extbsputil.h"
 #include "extbsppcb.h"
 
-#include "../../crypto/crypto.h"
+#if (PCB_DEBUGGING == 1)
+extern char		gMsg[];		/*	Debug message buffer.	*/
+#endif
+
+#include "crypto.h"
 
 
 /*****************************************************************************
@@ -153,7 +157,7 @@ int  bsp_pcbCopy(ExtensionBlock *newBlk, ExtensionBlock *oldBlk)
    int result = -1;
 
    PCB_DEBUG_PROC("+ bsp_pcbCopy(%x, %x)",
-                  (unsigned long) blk, (unsigned long) bundle);
+                  (unsigned long) newBlk, (unsigned long) oldBlk);
 
    CHKERR(newBlk);
    CHKERR(oldBlk);
@@ -283,7 +287,7 @@ int bsp_pcbCheck(AcqExtBlock *blk, AcqWorkArea *wk)
     BspPayloadReplaceKit *bprk;
     unsigned char *ltKeyValue = NULL, *sessionKeyValue = NULL;
     unsigned char *decryptedData = NULL;
-    unsigned long ltKeyLen = 0, sessionKeyLen = 0;
+    unsigned int ltKeyLen = 0, sessionKeyLen = 0;
     char *srcNode, *destNode;
     Sdr bpSdr = getIonsdr();
     Bundle *bundle = &(wk->bundle);
@@ -410,10 +414,14 @@ int bsp_pcbCheck(AcqExtBlock *blk, AcqWorkArea *wk)
     }
 
     // Finally, replace the old bundle zco object, with our new one with decrypted payload
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
     zco_destroy(bpSdr, bundle->payload.content);
+    if (sdr_end_xn(bpSdr) < 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+    }
+
     bundle->payload.content = bprk->newBundle;
-    sdr_end_xn(bpSdr);
     MRELEASE(bprk);
 
     // success, remove the block
@@ -458,8 +466,8 @@ int bsp_pcbProcessOnDequeue(ExtensionBlock *blk,
    unsigned char *sessionKeyValue = NULL;
    unsigned char *encryptedData = NULL;
    char *srcNode = NULL, *destNode = NULL;
-   unsigned long ltKeyLen = 0;
-   unsigned long sessionKeyLen = 0;
+   unsigned int ltKeyLen = 0;
+   unsigned int sessionKeyLen = 0;
    Sdnv sessionKeySdnv;
    int result = 0;
 
@@ -623,13 +631,17 @@ as expected.", NULL);
     }
 
     // Replace payload data with encrypted data in place, after destroying original payload data
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
     zco_destroy(bpSdr, bundle->payload.content);
+    if (sdr_end_xn(bpSdr) < 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+    }
+
     bundle->payload.content = encryptedPayloadZco;
-    sdr_end_xn(bpSdr);
 
     /* Serialize the Abstract Security Block. */
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
     raw_asb = bsp_serializeASB(&(blk->dataLength), &(asb));
 
     if((raw_asb == NULL) || (blk->dataLength == 0))
@@ -644,8 +656,12 @@ as expected.", NULL);
     }
 
     // Store the updated ASB for this block
-    sdr_write(bpSdr, blk->object, (char *)&(bundle->payload.content), sizeof(unsigned long));
-    sdr_end_xn(bpSdr);
+    sdr_write(bpSdr, blk->object, (char *)&(bundle->payload.content), sizeof(unsigned int));
+    if (sdr_end_xn(bpSdr) < 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+    }
+
     MRELEASE(raw_asb);
 
     // store this block in its bytes array. This is necessary as the
@@ -721,8 +737,8 @@ void    bsp_pcbRelease(ExtensionBlock *blk)
  *  09/13/11  R. Brown        Initial Implementation. 
  *****************************************************************************/
 
-BspPayloadReplaceKit *bsp_pcbPrepReplaceKit(Object bundle, unsigned long payloadLen, 
-                                        unsigned long headerLen, unsigned long trailerLen) 
+BspPayloadReplaceKit *bsp_pcbPrepReplaceKit(Object bundle, unsigned int payloadLen, 
+                                        unsigned int headerLen, unsigned int trailerLen) 
 {
     BspPayloadReplaceKit *bprk = MTAKE(sizeof(BspPayloadReplaceKit));
     CHKNULL(bprk);
@@ -757,13 +773,13 @@ BspPayloadReplaceKit *bsp_pcbPrepReplaceKit(Object bundle, unsigned long payload
  *****************************************************************************/
 
 int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
-                                     unsigned char *sessionKeyValue, unsigned long sessionKeyLen)   
+                                     unsigned char *sessionKeyValue, unsigned int sessionKeyLen)   
 {
     Object newSdrAddr = 0;
     int temp = 0;
-    unsigned long bundleLen = bprk->payloadLen +  
+    unsigned int bundleLen = bprk->payloadLen +  
                               bprk->headerLen + bprk->trailerLen;
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
     // Put the header onto a new payload object, though it is actually
     // going to be in the "source" portion
     if(bprk->headerLen > 0) {
@@ -772,7 +788,7 @@ int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
             // Error, cleanup
             BSP_DEBUG_ERR("bsp_pcbConstructDecryptedPayload: unable to allocate more\
                            sdr memory for header, len %d", bprk->headerLen);
-            sdr_end_xn(bpSdr);
+            sdr_cancel_xn(bpSdr);
             return -1;
         }
         sdr_write(bpSdr, newSdrAddr, (char *)bprk->headerBuff, bprk->headerLen);
@@ -783,7 +799,12 @@ int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
     {
         bprk->newBundle = zco_create(bpSdr, ZcoSdrSource, 0, 0, 0);
     }
-    sdr_end_xn(bpSdr);
+
+    if (sdr_end_xn(bpSdr) < 0 || bprk->newBundle == (Object) ERROR
+    || bprk->newBundle == 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+    }
 
     // Decrypt payload and append the references to resulting file onto our new object
     if((bsp_pcbCryptPayload(&bprk->newBundle, bprk->oldBundle,
@@ -791,13 +812,14 @@ int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
                             bprk->payloadLen, sessionKeyValue,
                             sessionKeyLen)) < 0)
     {
+        CHKERR(sdr_begin_xn(bpSdr));
         zco_destroy(bpSdr, bprk->newBundle);
+	oK(sdr_end_xn(bpSdr));
         PCB_DEBUG_ERR("x bsp_pcbConstructDecryptedPayload: Unable to decrypt the payload.", NULL);
-        sdr_begin_xn(bpSdr);
         return -1;
     }
 
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
     // Finally, append the trailer on our now decrypted bundle zco blob
     // going to be in the "source" portion
     if(bprk->trailerLen > 0) 
@@ -805,14 +827,23 @@ int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
         if((newSdrAddr = sdr_malloc(bpSdr, bprk->trailerLen)) == 0)
         {
             zco_destroy(bpSdr, bprk->newBundle);
-            PCB_DEBUG_ERR("x bsp_pcbConstructDecryptedPayload: unable to allocate more\
-                          sdr memory for trailer, len %d", bprk->trailerLen);
-            sdr_end_xn(bpSdr);
+            PCB_DEBUG_ERR("x bsp_pcbConstructDecryptedPayload: unable to \
+allocate more sdr memory for trailer, len %d", bprk->trailerLen);
+            oK(sdr_end_xn(bpSdr));
             return -1;
         }
-        sdr_write(bpSdr, newSdrAddr, (char *)bprk->trailerBuff, bprk->trailerLen);
-        zco_append_extent(bpSdr, bprk->newBundle, ZcoSdrSource,
-                          newSdrAddr, 0, bprk->trailerLen);
+
+        sdr_write(bpSdr, newSdrAddr, (char *) bprk->trailerBuff,
+			bprk->trailerLen);
+        if (zco_append_extent(bpSdr, bprk->newBundle, ZcoSdrSource,
+                          newSdrAddr, 0, bprk->trailerLen) <= 0)
+	{
+            zco_destroy(bpSdr, bprk->newBundle);
+            PCB_DEBUG_ERR("x bsp_pcbConstructDecryptedPayload: unable to \
+append extent to trailer ZCO, len %d", bprk->trailerLen);
+            oK(sdr_end_xn(bpSdr));
+            return -1;
+	}
     }
 
     // Now our new object should have the whole bundle in the source portion
@@ -823,10 +854,16 @@ int bsp_pcbConstructDecryptedPayload(Sdr bpSdr, BspPayloadReplaceKit *bprk,
         zco_destroy(bpSdr, bprk->newBundle);
         PCB_DEBUG_ERR("x bsp_pcbConstructDecryptedPayload: Length of rebuilt bundle with decrypted data != \
              length of original bundle. It is %d should be %d", temp, bundleLen);
-        sdr_end_xn(bpSdr);
+        oK(sdr_end_xn(bpSdr));
         return -1;
     }
-    sdr_end_xn(bpSdr);
+
+    if (sdr_end_xn(bpSdr) < 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+	    return -1;
+    }
+
     PCB_DEBUG_PROC("- bsp_pcbConstructDecryptedPayload--> %d", 0);
     return 0;
 }
@@ -858,7 +895,7 @@ int bsp_pcbIsolatePayload(Sdr bpSdr, BspPayloadReplaceKit *bprk)
     // bundle in it.. we separate it into the various parts, grab the header, trailer
     // then isolate the payload.  We do this to put it back together
     // with decrypted payload later
-    sdr_begin_xn(bpSdr);
+    CHKERR(sdr_begin_xn(bpSdr));
 
     // grab the header & trailer
     zco_start_receiving(bprk->oldBundle, &reader);
@@ -870,7 +907,7 @@ int bsp_pcbIsolatePayload(Sdr bpSdr, BspPayloadReplaceKit *bprk)
         {
             PCB_DEBUG_ERR("x bsp_pcbIsolatePayload: Failed to assign memory to buffer and grab full header\nreceived %d bytes instead of needed %d",
 		bytesTransmitted, bprk->headerLen);
-            sdr_end_xn(bpSdr);
+            oK(sdr_end_xn(bpSdr));
             return -1;
         }
         zco_delimit_source(bpSdr, bprk->oldBundle, bprk->headerLen,
@@ -884,7 +921,7 @@ int bsp_pcbIsolatePayload(Sdr bpSdr, BspPayloadReplaceKit *bprk)
         {
             PCB_DEBUG_ERR("x bsp_pcbIsolatePayload: Failed to assign memory to buffer and grab full trailer\nreceived %d bytes instead of needed %d",
 		bytesTransmitted, bprk->trailerLen);
-            sdr_end_xn(bpSdr);
+            oK(sdr_end_xn(bpSdr));
             return -1;
         }
     }
@@ -898,10 +935,16 @@ int bsp_pcbIsolatePayload(Sdr bpSdr, BspPayloadReplaceKit *bprk)
         // Something went wrong, bail
         PCB_DEBUG_ERR("x bsp_pcbIsolatePayload: Failed to isolate payload data \
              length of zco %d, should be payload length of %d", temp, bprk->payloadLen);
-        sdr_end_xn(bpSdr);
+        oK(sdr_end_xn(bpSdr));
         return -1;
     }
-    sdr_end_xn(bpSdr);
+
+    if (sdr_end_xn(bpSdr) < 0)
+    {
+	    putErrmsg("Transaction failed.", NULL);
+	    return -1;
+    }
+
     PCB_DEBUG_PROC("- bsp_pcbIsolatePayload:--> %d", 0);
     return 0;
 }
@@ -937,9 +980,9 @@ int bsp_pcbIsolatePayload(Sdr bpSdr, BspPayloadReplaceKit *bprk)
  *****************************************************************************/
 
 int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,  
-                        unsigned long payloadLen,
+                        unsigned int payloadLen,
                         unsigned char *keyValue,
-                        unsigned long keyLen)
+                        unsigned int keyLen)
                   
 {
    Sdr bpSdr = getIonsdr();
@@ -947,9 +990,9 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
    Object fileRef = 0;
    ZcoReader dataReader;
    arc4_context arcContext;
-   unsigned long bytesRemaining = 0;
-   unsigned long chunkSize = PCB_ENCRYPTION_CHUNK_SIZE;
-   unsigned long bytesRetrieved = 0;
+   unsigned int bytesRemaining = 0;
+   unsigned int chunkSize = PCB_ENCRYPTION_CHUNK_SIZE;
+   unsigned int bytesRetrieved = 0;
 
    PCB_DEBUG_INFO("+ bsp_pcbCryptPayload(0x%x, %ld, %s %d)",
                   (unsigned long) payloadData,
@@ -964,7 +1007,7 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
    CHKERR(dataBuffer);
 
    /*   Prepare the data for processing. */
-   sdr_begin_xn(bpSdr);
+   CHKERR(sdr_begin_xn(bpSdr));
    zco_start_transmitting(payloadData, &dataReader);
 
    /* Setup the context for arc4. */
@@ -993,7 +1036,7 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
         PCB_DEBUG_ERR("x bsp_pcbCryptPayload: Read %d bytes, but expected %d.",
            bytesRetrieved, chunkSize);
 
-        sdr_end_xn(bpSdr);
+        oK(sdr_end_xn(bpSdr));
 
         PCB_DEBUG_PROC("- bsp_pcbCryptPayload--> %d", -1);
         MRELEASE(dataBuffer);
@@ -1005,7 +1048,13 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
      arc4_crypt(&arcContext, bytesRetrieved, dataBuffer, dataBuffer);
      #endif
 
-     sdr_end_xn(bpSdr);
+     if (sdr_end_xn(bpSdr) < 0)
+     {
+	     putErrmsg("Transaction failed.", NULL);
+             MRELEASE(dataBuffer);
+	     return -1;
+     }
+
      // Transfer chunk into file and to our resultObject        
      if(transferToZcoFileSource(bpSdr, resultZco, &fileRef, fname, 
                                 (char *)dataBuffer, (int) bytesRetrieved) < 0)
@@ -1014,12 +1063,12 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
         MRELEASE(dataBuffer); 
         return -1;
      }
-     sdr_begin_xn(bpSdr);
+     CHKERR(sdr_begin_xn(bpSdr));
 
      bytesRemaining -= bytesRetrieved;
    }
 
-   sdr_end_xn(bpSdr);
+   oK(sdr_end_xn(bpSdr));
    MRELEASE(dataBuffer);
   
    return 0;
@@ -1051,9 +1100,9 @@ int bsp_pcbCryptPayload(Object *resultZco, Object payloadData, char *fname,
  *****************************************************************************/
 
 unsigned char *bsp_pcbCryptSessionKey(unsigned char *sessionKey, 
-                                      unsigned long sessionKeyLen, 
+                                      unsigned int sessionKeyLen, 
                                       unsigned char *ltKeyValue,
-                                      unsigned long ltKeyLen) 
+                                      unsigned int ltKeyLen) 
 {
     unsigned char *encryptedKey = MTAKE(sessionKeyLen); 
     CHKNULL(encryptedKey);
@@ -1088,7 +1137,7 @@ unsigned char *bsp_pcbCryptSessionKey(unsigned char *sessionKey,
  *  08/1/11  R. Brown        Initial Implementation.
  *****************************************************************************/
 
-unsigned char *bsp_pcbGenSessionKey(unsigned long *sessionKeyLen) 
+unsigned char *bsp_pcbGenSessionKey(unsigned int *sessionKeyLen) 
 {
     int i = 0;
     char possibleChar[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";//-_+=[]{};:\\|'\",.<>?/!@#$%^&*()~`";
