@@ -1,7 +1,11 @@
 #include "extbsputil.h"
 #include "extbsppib.h"
 
-#include "../../crypto/crypto.h"
+#if (PIB_DEBUGGING == 1)
+extern char		gMsg[];		/*	Debug message buffer.	*/
+#endif
+
+#include "crypto.h"
 
 
 /*****************************************************************************
@@ -151,8 +155,8 @@ int bsp_pibCopy(ExtensionBlock *newBlk, ExtensionBlock *oldBlk)
    BspAbstractSecurityBlock asb;
    int result = -1;
 
-   PIB_DEBUG_PROC("+ bsp_pibOffer(%x, %x)",
-                  (unsigned long) blk, (unsigned long) bundle);
+   PIB_DEBUG_PROC("+ bsp_pibCopy(%x, %x)",
+                  (unsigned long) newBlk, (unsigned long) oldBlk);
 
    CHKERR(newBlk);
    CHKERR(oldBlk);
@@ -283,7 +287,7 @@ int bsp_pibCheck(AcqExtBlock *blk, AcqWorkArea *wk)
    BspAbstractSecurityBlock *asb = NULL;
    int temp = 0, retval = 0;
    unsigned char   *digest;
-   unsigned long   digestLen;
+   unsigned int   digestLen;
 
    PIB_DEBUG_PROC("+ bsp_pibCheck(%x, %x)",
                   (unsigned long) blk, (unsigned long) wk);
@@ -365,7 +369,7 @@ int bsp_pibCheck(AcqExtBlock *blk, AcqWorkArea *wk)
    {
       /* Compare expected result to the computed digest. */
 	  unsigned char   *tmpDigest;
-	  unsigned long   tmpDigestLen;
+	  unsigned int   tmpDigestLen;
 	  char *keyValueBuffer;
 	  int keyLen;
 	  BspSecurityInfo secInfo;
@@ -413,7 +417,7 @@ int bsp_pibCheck(AcqExtBlock *blk, AcqWorkArea *wk)
 
           // The entire bundle currently sits in payload.content,
           // extract just the real payload content
-          sdr_begin_xn(bpSdr);
+          CHKERR(sdr_begin_xn(bpSdr));
           payloadData = zco_clone(bpSdr, wk->bundle.payload.content, wk->headerLength,
                                   wk->bundle.payload.length);
           // The length should be == to the payload length now
@@ -423,21 +427,28 @@ int bsp_pibCheck(AcqExtBlock *blk, AcqWorkArea *wk)
               PIB_DEBUG_ERR("x bsp_pibCheck: Failed to isolate payload data \
                              length of zco %d, should be payload length of %d", temp, wk->bundle.payload.length);
               zco_destroy(bpSdr, payloadData);
-              sdr_end_xn(bpSdr);
+              oK(sdr_end_xn(bpSdr));
               PIB_DEBUG_PROC("- bsp_pibCheck(%d)", -1);
               return -1;
           }
 
-          sdr_end_xn(bpSdr);
+          if (sdr_end_xn(bpSdr) < 0)
+	  {
+		  putErrmsg("Transaction failed.", NULL);
+		  return -1;
+	  }
 
 	  /* Grab the digest of the received payload */
 	  tmpDigest = bsp_pibGetSecResult(payloadData, wk->bundle.payload.length,
 			                  keyValueBuffer, keyLen, &tmpDigestLen);
 
           // Destroy the payload object as we don't need it any longer
-          sdr_begin_xn(bpSdr);
+          CHKERR(sdr_begin_xn(bpSdr));
           zco_destroy(bpSdr, payloadData);
-          sdr_end_xn(bpSdr);
+          if (sdr_end_xn(bpSdr) < 0)
+	  {
+		  putErrmsg("Transaction failed.", NULL);
+	  }
 
       cmpResult = memcmp(digest, tmpDigest, digestLen);
 
@@ -493,12 +504,12 @@ int bsp_pibProcessOnDequeue(ExtensionBlock *blk, Bundle *bundle,
    char *keyValueBuffer = NULL;
    int keyLen = 0;
    unsigned char *digest;
-   unsigned long digestLen = 0;
+   unsigned int digestLen = 0;
    char *srcNode = NULL, *destNode = NULL;
    Lyst eidRefs = NULL;
    unsigned char *raw_asb;
    Sdnv digestSdnv;
-   unsigned long digestOffset = 0;
+   unsigned int digestOffset = 0;
 
    PIB_DEBUG_PROC("+ bsp_pibProcessOnDequeue(%x, %x, %x)",
                   (unsigned long) blk,
@@ -720,10 +731,10 @@ void    bsp_pibRelease(ExtensionBlock *blk)
  *****************************************************************************/
 
 unsigned char *bsp_pibGetSecResult(Object dataObj,
-                                   unsigned long dataLen,
+                                   unsigned int dataLen,
                                    char *keyValue,
-                                   unsigned long keyLen,
-                                   unsigned long *hashLen)
+                                   unsigned int keyLen,
+                                   unsigned int *hashLen)
 {
    Sdr bpSdr = getIonsdr();
    unsigned char *hashData = NULL;
@@ -732,9 +743,9 @@ unsigned char *bsp_pibGetSecResult(Object dataObj,
    ZcoReader dataReader;
    char *authContext;
    int authCtxLen = 0;
-   unsigned long bytesRemaining = 0;
-   unsigned long chunkSize = BSP_PIB_BLOCKING_SIZE;
-   unsigned long bytesRetrieved = 0;
+   unsigned int bytesRemaining = 0;
+   unsigned int chunkSize = BSP_PIB_BLOCKING_SIZE;
+   unsigned int bytesRetrieved = 0;
 
    PIB_DEBUG_INFO("+ bsp_pibGetSecResult(0x%x, %ld, %s %d, 0x%x)",
                   (unsigned long) dataObj,
@@ -775,7 +786,7 @@ unsigned char *bsp_pibGetSecResult(Object dataObj,
    }
 
    /*   Prepare the data for processing. */
-   sdr_begin_xn(bpSdr);
+   CHKNULL(sdr_begin_xn(bpSdr));
    zco_start_transmitting(dataObj, &dataReader);
    
    hmac_sha256_init(authContext,(unsigned char *)keyValue, keyLen);
@@ -803,7 +814,7 @@ unsigned char *bsp_pibGetSecResult(Object dataObj,
 					   bytesRetrieved, chunkSize);
 
     	 MRELEASE(authContext);
-         sdr_end_xn(bpSdr);
+         oK(sdr_end_xn(bpSdr));
 
          *hashLen = 0;
          PIB_DEBUG_PROC("- bsp_pibGetSecResult--> NULL", NULL);
@@ -829,7 +840,7 @@ unsigned char *bsp_pibGetSecResult(Object dataObj,
                      BSP_PIB_HMAC_SHA256_RESULT_LEN);
 
       MRELEASE(authContext);
-      sdr_end_xn(bpSdr);
+      oK(sdr_end_xn(bpSdr));
 
       *hashLen = 0;
       PIB_DEBUG_PROC("- bsp_pibGetSecResult--> NULL", NULL);
@@ -848,8 +859,7 @@ unsigned char *bsp_pibGetSecResult(Object dataObj,
    MRELEASE(authContext);
    if ((i = sdr_end_xn(bpSdr)) < 0)
    {
-      PIB_DEBUG_ERR("x bsp_pibGetSecResult: Failed closing transaction. Result is %d.",
-                     i);
+      PIB_DEBUG_ERR("x bsp_pibGetSecResult: Failed closing transaction. Result is %d.", i);
 
       MRELEASE(hashData);
       *hashLen = 0;
