@@ -240,7 +240,7 @@ static void	*receiveBundles(void *parm)
 		 * Creating a thread to send out keep alives, which
 		 * makes the TCPCL bi directional
 		 */
-		if (pthread_create(&kthread, NULL, sendKeepalives, kparms))
+		if (pthread_begin(&kthread, NULL, sendKeepalives, kparms))
 		{
 			putSysErrmsg("tcpcli can't create new thread for \
 keepalives", NULL);
@@ -403,20 +403,13 @@ thread", NULL);
 		memcpy((char *) &hostNbr,
 				(char *) &(fromAddr->sin_addr.s_addr), 4);
 		hostNbr = ntohl(hostNbr);
-		if (getInternetHostName(hostNbr, hostName))
-		{
-			parms->senderEid = parms->senderEidBuffer;
-			getSenderEid(&(parms->senderEid), hostName);
-		}
-		else
-		{
-			parms->senderEid = NULL;
-		}
-
+		printDottedString(hostNbr, hostName);
+		parms->senderEid = parms->senderEidBuffer;
+		getSenderEid(&(parms->senderEid), hostName);
 		parms->cloSocketName = cloSocketName;
 		parms->cliRunning = &(atp->running);
                 parms->receiveRunning = 1;
-		if (pthread_create(&(parms->thread), NULL, receiveBundles,
+		if (pthread_begin(&(parms->thread), NULL, receiveBundles,
 					parms))
 		{
 			putSysErrmsg("tcpcli can't create new thread", NULL);
@@ -526,9 +519,11 @@ int	main(int argc, char *argv[])
 	/*	All command-line arguments are now validated.		*/
 
 	sdr = getIonsdr();
+	CHKERR(sdr_begin_xn(sdr));
 	sdr_read(sdr, (char *) &duct, sdr_list_data(sdr, vduct->inductElt),
 			sizeof(Induct));
 	sdr_read(sdr, (char *) &protocol, duct.protocol, sizeof(ClProtocol));
+	sdr_exit_xn(sdr);
 	if (protocol.nominalRate == 0)
 	{
 		vduct->acqThrottle.nominalRate = DEFAULT_TCP_RATE;
@@ -608,13 +603,13 @@ int	main(int argc, char *argv[])
 	ionNoteMainThread("tcpcli");
 	isignal(SIGTERM, interruptThread);
 #ifndef mingw
-        isignal(SIGPIPE, SIG_IGN); //Ignore pipe break and handle it gracefully
+	isignal(SIGPIPE, SIG_IGN);
 #endif
 
 	/*	Start the access thread.				*/
 
 	atp.running = 1;
-	if (pthread_create(&accessThread, NULL, spawnReceivers, &atp))
+	if (pthread_begin(&accessThread, NULL, spawnReceivers, &atp))
 	{
 		closesocket(atp.ductSocket);
 		putSysErrmsg("tcpcli can't create access thread", NULL);
