@@ -89,16 +89,9 @@ static void	*handleDatagrams(void *parm)
 		memcpy((char *) &hostNbr,
 				(char *) &(fromAddr.sin_addr.s_addr), 4);
 		hostNbr = ntohl(hostNbr);
-		if (getInternetHostName(hostNbr, hostName))
-		{
-			senderEid = senderEidBuffer;
-			getSenderEid(&senderEid, hostName);
-		}
-		else
-		{
-			senderEid = NULL;
-		}
-
+		printDottedString(hostNbr, hostName);
+		senderEid = senderEidBuffer;
+		getSenderEid(&senderEid, hostName);
 		if (bpBeginAcq(work, 0, senderEid) < 0
 		|| bpContinueAcq(work, buffer, bundleLength) < 0
 		|| bpEndAcq(work) < 0)
@@ -181,9 +174,11 @@ int	main(int argc, char *argv[])
 	/*	All command-line arguments are now validated.		*/
 
 	sdr = getIonsdr();
+	CHKZERO(sdr_begin_xn(sdr));
 	sdr_read(sdr, (char *) &duct, sdr_list_data(sdr, vduct->inductElt),
 			sizeof(Induct));
 	sdr_read(sdr, (char *) &protocol, duct.protocol, sizeof(ClProtocol));
+	sdr_exit_xn(sdr);
 	if (protocol.nominalRate == 0)
 	{
 		vduct->acqThrottle.nominalRate = DEFAULT_UDP_RATE;
@@ -243,7 +238,7 @@ int	main(int argc, char *argv[])
 	/*	Start the receiver thread.				*/
 
 	rtp.running = 1;
-	if (pthread_create(&receiverThread, NULL, handleDatagrams, &rtp))
+	if (pthread_begin(&receiverThread, NULL, handleDatagrams, &rtp))
 	{
 		closesocket(rtp.ductSocket);
 		putSysErrmsg("udpcli can't create receiver thread", NULL);
