@@ -93,9 +93,11 @@ int	main(int argc, char *argv[])
 	/*	All command-line arguments are now validated.		*/
 
 	sdr = getIonsdr();
+	CHKZERO(sdr_begin_xn(sdr));
 	sdr_read(sdr, (char *) &outduct, sdr_list_data(sdr, vduct->outductElt),
 			sizeof(Outduct));
 	sdr_read(sdr, (char *) &protocol, outduct.protocol, sizeof(ClProtocol));
+	sdr_exit_xn(sdr);
 	if (protocol.nominalRate == 0)
 	{
 		vduct->xmitThrottle.nominalRate = DEFAULT_UDP_RATE;
@@ -147,7 +149,7 @@ int	main(int argc, char *argv[])
 		{
 			writeMemoNote("[?] Can't get IP address for host",
 					destDuctName);
-			sdr_begin_xn(sdr);
+			CHKZERO(sdr_begin_xn(sdr));
 			zco_destroy(sdr, bundleZco);
 			if (sdr_end_xn(sdr) < 0)
 			{
@@ -165,7 +167,9 @@ int	main(int argc, char *argv[])
 		inetName->sin_port = portNbr;
 		memcpy((char *) &(inetName->sin_addr.s_addr),
 				(char *) &hostNbr, 4);
+		CHKZERO(sdr_begin_xn(sdr));
 		bundleLength = zco_length(sdr, bundleZco);
+		sdr_exit_xn(sdr);
 		bytesSent = sendBundleByUDP(&socketName, &ductSocket,
 				bundleLength, bundleZco, buffer);
 		if (bytesSent < bundleLength)
