@@ -12,48 +12,39 @@
 
 #define	BPRECVBUFSZ	(65536)
 
-static BpSAP	_bpsap(BpSAP *newSap)
+typedef struct
 {
-	void	*value;
 	BpSAP	sap;
+	int	running;
+} BptestState;
 
-	if (newSap)			/*	Add task variable.	*/
+static BptestState	*_bptestState(BptestState *newState)
+{
+	void		*value;
+	BptestState	*state;
+
+	if (newState)			/*	Add task variable.	*/
 	{
-		value = (void *) (*newSap);
-		sap = (BpSAP) sm_TaskVar(&value);
+		value = (void *) (newState);
+		state = (BptestState *) sm_TaskVar(&value);
 	}
 	else				/*	Retrieve task variable.	*/
 	{
-		sap = (BpSAP) sm_TaskVar(NULL);
+		state = (BptestState *) sm_TaskVar(NULL);
 	}
 
-	return sap;
-}
-
-static int	_running(int *newState)
-{
-	void	*value = NULL;
-	BpSAP	sap;
-
-	if (newState)			/*	Only used for Stop.	*/
-	{
-		sap = (BpSAP) sm_TaskVar(&value);
-	}
-	else				/*	Retrieve task variable.	*/
-	{
-		sap = (BpSAP) sm_TaskVar(NULL);
-	}
-
-	return (sap == NULL ? 0 : 1);
+	return state;
 }
 
 static void	handleQuit()
 {
-	int	stop = 0;
+	BptestState	*state;
 
+	isignal(SIGINT, handleQuit);
 	writeMemo("[i] bprecvfile interrupted.");
-	bp_interrupt(_bpsap(NULL));
-	oK(_running(&stop));
+	state = _bptestState(NULL);
+	bp_interrupt(state->sap);
+	state->running = 0;
 }
 
 static int	receiveFile(Sdr sdr, BpDelivery *dlv)
@@ -80,7 +71,7 @@ static int	receiveFile(Sdr sdr, BpDelivery *dlv)
 
 	zco_start_receiving(dlv->adu, &reader);
 	remainingLength = contentLength;
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	while (remainingLength > 0)
 	{
 		recvLength = BPRECVBUFSZ;
@@ -127,19 +118,21 @@ int	bprecvfile(int a1, int a2, int a3, int a4, int a5,
 		int a6, int a7, int a8, int a9, int a10)
 {
 	char		*ownEid = (char *) a1;
+	int		maxFiles = strtol((char *) a1, NULL, 0);
 #else
 int	main(int argc, char **argv)
 {
 	char		*ownEid = (argc > 1 ? argv[1] : NULL);
+	int		maxFiles = (argc > 2 ? strtol(argv[2], NULL, 0) : 0);
 #endif
-	BpSAP		sap;
+	BptestState	state = { NULL, 1 };
 	Sdr		sdr;
 	BpDelivery	dlv;
-	int		stop = 0;
+	int		filesReceived = 0;
 
 	if (ownEid == NULL)
 	{
-		PUTS("Usage: bprecvfile <own endpoint ID>");
+		PUTS("Usage: bprecvfile <own endpoint ID> [<max # files>]");
 		return 0;
 	}
 
@@ -149,36 +142,37 @@ int	main(int argc, char **argv)
 		return -1;
 	}
 
-	if (bp_open(ownEid, &sap) < 0)
+	if (bp_open(ownEid, &state.sap) < 0)
 	{
 		putErrmsg("Can't open own endpoint.", ownEid);
 		return -1;
 	}
 
-	oK(_bpsap(&sap));
+	oK(_bptestState(&state));
 	sdr = bp_get_sdr();
 	isignal(SIGINT, handleQuit);
 	writeMemo("[i] bprecvfile is running.");
-	while (_running(NULL))
+	while (state.running)
 	{
-		if (bp_receive(sap, &dlv, BP_BLOCKING) < 0)
+		if (bp_receive(state.sap, &dlv, BP_BLOCKING) < 0)
 		{
 			putErrmsg("bprecvfile bundle reception failed.", NULL);
-			oK(_running(&stop));
+			state.running = 0;
 			continue;
 		}
 
 		switch (dlv.result)
 		{
 		case BpEndpointStopped:
-			oK(_running(&stop));
+			state.running = 0;
 			break;		/*	Out of switch.		*/
 
 		case BpPayloadPresent:
+			filesReceived++;
 			if (receiveFile(sdr, &dlv) < 0)
 			{
 				putErrmsg("bprecvfile cannot continue.", NULL);
-				oK(_running(&stop));
+				state.running = 0;
 			}
 
 			/*	Intentional fall-through to default.	*/
@@ -187,10 +181,19 @@ int	main(int argc, char **argv)
 			break;		/*	Out of switch.		*/
 		}
 
+		if (maxFiles > 0)
+		{
+			if (filesReceived == maxFiles)
+			{
+				writeMemo("[i] bprecvfile has reached limit.");
+				state.running = 0;
+			}
+		}
+
 		bp_release_delivery(&dlv, 1);
 	}
 
-	bp_close(sap);
+	bp_close(state.sap);
 	writeErrmsgMemos();
 	writeMemo("[i] Stopping bprecvfile.");
 	bp_detach();
