@@ -86,7 +86,7 @@ static IonNeighbor	*getNeighbor(IonVdb *vdb, unsigned long nodeNbr)
 	return neighbor;
 }
 
-static int	dispatchEvent(IonVdb *vdb, IonEvent *event)
+static int	dispatchEvent(IonVdb *vdb, IonEvent *event, int *forecastNeeded)
 {
 	PsmPartition	ionwm = getIonwm();
 	PsmAddress	addr;
@@ -113,6 +113,7 @@ static int	dispatchEvent(IonVdb *vdb, IonEvent *event)
 			if (neighbor)
 			{
 				neighbor->xmitRate = 0;
+				*forecastNeeded = 1;
 			}
 		}
 
@@ -139,6 +140,7 @@ static int	dispatchEvent(IonVdb *vdb, IonEvent *event)
 			if (neighbor)
 			{
 				neighbor->recvRate = 0;
+				*forecastNeeded = 1;
 			}
 		}
 
@@ -175,6 +177,7 @@ static int	dispatchEvent(IonVdb *vdb, IonEvent *event)
 			if (neighbor)
 			{
 				neighbor->xmitRate = cxref->xmitRate;
+				*forecastNeeded = 1;
 			}
 		}
 
@@ -279,6 +282,7 @@ static int	dispatchEvent(IonVdb *vdb, IonEvent *event)
 			if (neighbor)
 			{
 				neighbor->recvRate = cxref->xmitRate;
+				*forecastNeeded = 1;
 			}
 		}
 
@@ -314,6 +318,7 @@ int	main(int argc, char *argv[])
 	IonProbe	*probe;
 	int		destNodeNbr;
 	int		neighborNodeNbr;
+	int		forecastNeeded;
 	IonEvent	*event;
 
 	if (ionAttach() < 0)
@@ -339,7 +344,12 @@ int	main(int argc, char *argv[])
 
 		snooze(1);
 		currentTime = getUTCTime();
-		sdr_begin_xn(sdr);
+		if (!sdr_begin_xn(sdr))
+		{
+			putErrmsg("rfxclock failed to begin new transaction.",
+					NULL);
+			break;
+		}
 
 		/*	First enable probes.				*/
 
@@ -348,9 +358,16 @@ int	main(int argc, char *argv[])
 		{
 			addr = sm_list_data(ionwm, elt);
 			probe = (IonProbe *) psp(ionwm, addr);
-			CHKZERO(probe);
+			if (probe == NULL)
+			{
+				putErrmsg("List of probes is corrupt.", NULL);
+				sdr_exit_xn(sdr);
+				break;
+			}
+
 			if (probe->time > currentTime)
 			{
+				sdr_exit_xn(sdr);
 				break;	/*	No more for now.	*/
 			}
 
@@ -373,6 +390,7 @@ int	main(int argc, char *argv[])
 		 *	so doing, adjust the volatile contact and
 		 *	range state of the local node.			*/
 
+		forecastNeeded = 0;
 		while (1)
 		{
 			elt = sm_rbt_first(ionwm, vdb->timeline);
@@ -388,7 +406,7 @@ int	main(int argc, char *argv[])
 				break;
 			}
 
-			if (dispatchEvent(vdb, event) < 0)
+			if (dispatchEvent(vdb, event, &forecastNeeded) < 0)
 			{
 				putErrmsg("Failed handling event.", NULL);
 				break;
@@ -403,6 +421,11 @@ int	main(int argc, char *argv[])
 			putErrmsg("Can't set current topology.", NULL);
 			return -1;
 		}
+
+		if (forecastNeeded)
+		{
+			oK(pseudoshell("ionwarn"));
+		}
 	}
 
 	writeErrmsgMemos();
