@@ -23,7 +23,7 @@ typedef struct
 	char		*wmName;
 } sm_WmParms;
 
-typedef long             sm_SemId;
+typedef int		sm_SemId;
 #define SM_SEM_NONE	(-1)
 
 #define SM_SEM_FIFO	0
@@ -33,6 +33,18 @@ typedef long             sm_SemId;
 
 #define	ICI_PRIORITY	250
 
+#if defined (bionic) || defined (uClibc)
+extern int		sm_BeginPthread(pthread_t *threadId,
+				const pthread_attr_t *attr,
+				void *(*function)(void *), void *arg);
+#define pthread_begin(w,x,y,z) sm_BeginPthread(w, x, y, z)
+extern void		sm_EndPthread(pthread_t threadId);
+#define pthread_end(x)	sm_EndPthread(x)
+#else
+#define pthread_begin(w,x,y,z)	pthread_create(w, x, y, z)
+#define pthread_end(x)		pthread_cancel(x)
+#endif
+
 /*      IPC services access control */
 extern int		sm_ipc_init();
 extern void		sm_ipc_stop();
@@ -43,7 +55,7 @@ extern int		sm_GetUniqueKey();
 extern sm_SemId		sm_SemCreate(int key, int semType);
 extern int		sm_SemTake(sm_SemId semId);
 extern void		sm_SemGive(sm_SemId semId);
-extern int		sm_SemUnwedge(sm_SemId semId, int interval);
+extern int		sm_SemUnwedge(sm_SemId semId, int timeoutSeconds);
 extern void		sm_SemDelete(sm_SemId semId);
 extern void		sm_SemEnd(sm_SemId semId);
 extern int		sm_SemEnded(sm_SemId semId);
