@@ -36,12 +36,31 @@ static SecDB	*_secConstants()
 {
 	static SecDB	buf;
 	static SecDB	*db = NULL;
+	Sdr		sdr;
+	Object		dbObject;
 
 	if (db == NULL)
 	{
-		sdr_read(getIonsdr(), (char *) &buf, _secdbObject(0),
-				sizeof(SecDB));
-		db = &buf;
+		sdr = getIonsdr();
+		CHKNULL(sdr);
+		dbObject = _secdbObject(0);
+		if (dbObject)
+		{
+			if (sdr_heap_is_halted(sdr))
+			{
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(SecDB));
+			}
+			else
+			{
+				CHKNULL(sdr_begin_xn(sdr));
+				sdr_read(sdr, (char *) &buf, dbObject,
+						sizeof(SecDB));
+				sdr_exit_xn(sdr);
+			}
+
+			db = &buf;
+		}
 	}
 
 	return db;
@@ -60,7 +79,7 @@ int	secInitialize()
 	}
 
 	ionsdr = getIonsdr();
-	sdr_begin_xn(ionsdr);
+	CHKERR(sdr_begin_xn(ionsdr));
 	secdbObject = sdr_find(ionsdr, _secDbName(), NULL);
 	switch (secdbObject)
 	{
@@ -119,7 +138,7 @@ int	secAttach()
 	secdbObject = _secdbObject(NULL);
 	if (secdbObject == 0)
 	{
-		sdr_begin_xn(ionsdr);
+		CHKERR(sdr_begin_xn(ionsdr));
 		secdbObject = sdr_find(ionsdr, _secDbName(), NULL);
 		sdr_exit_xn(ionsdr);
 		if (secdbObject == 0)
@@ -227,19 +246,14 @@ void	ionClear(char *srcEid, char *destEid, char *blockType)
         CHKVOID(srcEid);
         CHKVOID(destEid);
         CHKVOID(blockType);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] ionClear can't find ION security.");
-		return;
-	}
-
+	CHKVOID(secdb);
 	srcEidLen = istrlen(srcEid, SDRSTRING_BUFSZ);
 	destEidLen = istrlen(destEid, SDRSTRING_BUFSZ);
        	if ((blockType[0] == '~') || (bspTypeToInt(blockType) == BSP_BAB_TYPE))
        	{
         	// For each bab rule, if src/dest match, delete it.  
          	OBJ_POINTER(BspBabRule, rule);
-		sdr_begin_xn(sdr);
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sdr_list_first(sdr, secdb->bspBabRules); elt;
 			elt = temp)
 		{
@@ -281,7 +295,7 @@ void	ionClear(char *srcEid, char *destEid, char *blockType)
         {
                 // For each pib rule, if src/dest match, delete it.  
                 OBJ_POINTER(BspPibRule, rule);
-                sdr_begin_xn(sdr);
+                CHKVOID(sdr_begin_xn(sdr));
                 for (elt = sdr_list_first(sdr, secdb->bspPibRules); elt;
                         elt = temp)
                 {
@@ -322,7 +336,7 @@ void	ionClear(char *srcEid, char *destEid, char *blockType)
         {
                 // For each pib rule, if src/dest match, delete it.  
                 OBJ_POINTER(BspPcbRule, rule);
-                sdr_begin_xn(sdr);
+                CHKVOID(sdr_begin_xn(sdr));
                 for (elt = sdr_list_first(sdr, secdb->bspPcbRules); elt;
                         elt = temp)
                 {
@@ -382,6 +396,11 @@ static Object	locateKey(char *keyName, Object *nextKey)
 
 	CHKZERO(ionLocked());
 	if (nextKey) *nextKey = 0;	/*	Default.		*/
+	if (secdb == NULL)	/*	No security database declared.	*/
+	{
+		return 0;
+	}
+
 	for (elt = sdr_list_first(sdr, secdb->keys); elt;
 			elt = sdr_list_next(sdr, elt))
 	{
@@ -416,13 +435,7 @@ void	sec_findKey(char *keyName, Object *keyAddr, Object *eltp)
 	CHKVOID(keyAddr);
 	CHKVOID(eltp);
 	*eltp = 0;
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_findKey can't find ION security.");
-		return;
-	}
-
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	elt = locateKey(keyName, NULL);
 	if (elt == 0)
 	{
@@ -502,6 +515,7 @@ int	sec_addKey(char *keyName, char *fileName)
 
 	CHKERR(keyName);
 	CHKERR(fileName);
+	CHKERR(secdb);
 	if (*keyName == '\0' || istrlen(keyName, 32) > 31)
 	{
 		writeMemoNote("[?] Invalid key name", keyName);
@@ -514,13 +528,7 @@ int	sec_addKey(char *keyName, char *fileName)
 		return 0;
 	}
 
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_addKey can't find ION security.");
-		return 0;
-	}
-
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	if (locateKey(keyName, &nextKey) != 0)
 	{
 		sdr_exit_xn(sdr);
@@ -594,13 +602,7 @@ int	sec_updateKey(char *keyName, char *fileName)
 		return 0;
 	}
 
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_updateKey can't find ION security.");
-		return 0;
-	}
-
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateKey(keyName, NULL);
 	if (elt == 0)
 	{
@@ -648,13 +650,7 @@ int	sec_removeKey(char *keyName)
 		OBJ_POINTER(SecKey, key);
 
 	CHKERR(keyName);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_removeKey can't find ION security.");
-		return 0;
-	}
-
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	elt = locateKey(keyName, NULL);
 	if (elt == 0)
 	{
@@ -690,13 +686,7 @@ int	sec_get_key(char *keyName, int *keyBufferLength, char *keyValueBuffer)
 	CHKERR(keyName);
 	CHKERR(keyBufferLength);
 	CHKERR(keyValueBuffer);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_get_key can't find ION security.");
-		return 0;
-	}
-
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	sec_findKey(keyName, &keyAddr, &elt);
 	if (elt == 0)
 	{
@@ -771,8 +761,10 @@ static int	eidsMatch(char *firstEid, int firstEidLen, char *secondEid,
 	int	firstPos = -1;
 	int	secondPos = -1;
 
-	CHKERR(firstEid);
-	CHKERR(secondEid);
+	CHKZERO(firstEid);
+	CHKZERO(firstEidLen > 0);
+	CHKZERO(secondEid);
+	CHKZERO(secondEidLen > 0);
 
 	/*
 	 * First, determine if (and the pos of) end-of-line wildcards.
@@ -841,13 +833,12 @@ int	sec_get_bspBabRule(char *srcEid, char *destEid, Object *ruleAddr,
 	CHKERR(ruleAddr);
 	CHKERR(eltp);
 	*eltp = 0;
-	if (secAttach() < 0)
+	if (secdb == NULL)	/*	No security database declared.	*/
 	{
-		writeMemo("[?] sec_get_bspBabRule can't find ION security.");
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	for (elt = sdr_list_first(sdr, secdb->bspBabRules); elt;
 			elt = sdr_list_next(sdr, elt))
 	{
@@ -885,12 +876,6 @@ int	sec_findBspBabRule(char *srcEid, char *destEid, Object *ruleAddr,
 	CHKERR(ruleAddr);
 	CHKERR(eltp);
 	*eltp = 0;
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_findBspBabRule can't find ION security.");
-		return -1;
-	}
-
 	if ((filterEid(srcEid, srcEid) == 0)
 	|| (filterEid(destEid, destEid) == 0))
 	{
@@ -915,13 +900,7 @@ int	sec_addBspBabRule(char *srcEid, char *destEid, char *ciphersuiteName,
 	CHKERR(destEid);
 	CHKERR(ciphersuiteName);
 	CHKERR(keyName);
-
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_addBspBabRule can't find ION security.");
-		return 0;
-	}
-
+	CHKERR(secdb);
 	if (istrlen(ciphersuiteName, 32) > 31)
 	{
 		writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -963,7 +942,7 @@ single sender endpoint, not all endpoints on the sending node", srcEid);
 single receiver endpoint, not all endpoints on the receiving node", destEid);
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	rule.securitySrcEid = sdr_string_create(sdr, srcEid);
 	rule.securityDestEid = sdr_string_create(sdr, destEid);
 	istrcpy(rule.ciphersuiteName, ciphersuiteName,
@@ -1006,12 +985,6 @@ int	sec_updateBspBabRule(char *srcEid, char *destEid, char *ciphersuiteName,
 	CHKERR(destEid);
 	CHKERR(ciphersuiteName);
 	CHKERR(keyName);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_updateBabRule can't find ION security.");
-		return 0;
-	}
-
 	if (istrlen(ciphersuiteName, 32) > 31)
 	{
 		writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -1038,7 +1011,7 @@ int	sec_updateBspBabRule(char *srcEid, char *destEid, char *ciphersuiteName,
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &rule, ruleObj, sizeof(BspBabRule));
 	istrcpy(rule.ciphersuiteName, ciphersuiteName,
 			sizeof rule.ciphersuiteName);
@@ -1062,12 +1035,6 @@ int	sec_removeBspBabRule(char *srcEid, char *destEid)
 
 	CHKERR(srcEid);
 	CHKERR(destEid);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_removeBspBabRule can't find ION security.");
-		return 0;
-	}
-
 	if ((filterEid(srcEid, srcEid) == 0)
 	|| (filterEid(destEid, destEid) == 0))
 	{
@@ -1082,7 +1049,7 @@ int	sec_removeBspBabRule(char *srcEid, char *destEid)
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	sdr_list_delete(sdr, elt, NULL, NULL);
 	GET_OBJ_POINTER(sdr, BspBabRule, rule, ruleObj);
 	sdr_free(sdr, rule->securitySrcEid);
@@ -1132,13 +1099,12 @@ int	sec_get_bspPibRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
 	CHKERR(ruleAddr);
 	CHKERR(eltp);
 	*eltp = 0;
-	if (secAttach() < 0)
+	if (secdb == NULL)	/*	No security database declared.	*/
 	{
-		writeMemo("[?] sec_get_bspPibRule can't find ION security.");
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	for (elt = sdr_list_first(sdr, secdb->bspPibRules); elt;
 			elt = sdr_list_next(sdr, elt))
 	{
@@ -1182,12 +1148,6 @@ int	sec_findBspPibRule(char *secSrcEid, char *secDestEid, int BlockTypeNbr,
 	CHKERR(ruleAddr);
 	CHKERR(eltp);
 	*eltp = 0;
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_findBspPibRule can't find ION security.");
-		return -1;
-	}
-
 	if ((filterEid(secSrcEid, secSrcEid) == 0)
 	|| (filterEid(secDestEid, secDestEid) == 0))
 	{
@@ -1211,12 +1171,7 @@ int	sec_addBspPibRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
 	CHKERR(secDestEid);
 	CHKERR(ciphersuiteName);
 	CHKERR(keyName);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_addBspPibRule can't find ION security.");
-		return 0;
-	}
-
+	CHKERR(secdb);
 	if (istrlen(ciphersuiteName, 32) > 31)
 	{
 		writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -1246,7 +1201,7 @@ int	sec_addBspPibRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
 
 	/*	Okay to add this rule to the database.			*/
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	rule.securitySrcEid = sdr_string_create(sdr, secSrcEid);
 	rule.securityDestEid = sdr_string_create(sdr, secDestEid);
 	rule.blockTypeNbr = blockTypeNbr;
@@ -1285,12 +1240,6 @@ int	sec_updateBspPibRule(char *secSrcEid, char *secDestEid,
 	CHKERR(secDestEid);
 	CHKERR(ciphersuiteName);
 	CHKERR(keyName);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_updatePibRule can't find ION security.");
-		return 0;
-	}
-
 	if (istrlen(ciphersuiteName, 32) > 31)
 	{
 		writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -1318,7 +1267,7 @@ int	sec_updateBspPibRule(char *secSrcEid, char *secDestEid,
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &rule, ruleObj, sizeof(BspPibRule));
 	istrcpy(rule.ciphersuiteName, ciphersuiteName,
 			sizeof rule.ciphersuiteName);
@@ -1343,12 +1292,6 @@ int	sec_removeBspPibRule(char *secSrcEid, char *secDestEid,
 
 	CHKERR(secSrcEid);
 	CHKERR(secDestEid);
-	if (secAttach() < 0)
-	{
-		writeMemo("[?] sec_removeBspPibRule can't find ION security.");
-		return 0;
-	}
-
 	if ((filterEid(secSrcEid, secSrcEid) == 0)
 	|| (filterEid(secDestEid, secDestEid) == 0))
 	{
@@ -1364,7 +1307,7 @@ int	sec_removeBspPibRule(char *secSrcEid, char *secDestEid,
 		return 0;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKERR(sdr_begin_xn(sdr));
 	sdr_list_delete(sdr, elt, NULL, NULL);
 	GET_OBJ_POINTER(sdr, BspPibRule, rule, ruleObj);
 	sdr_free(sdr, rule->securitySrcEid);
@@ -1399,13 +1342,12 @@ int    sec_get_bspPcbRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
         CHKERR(ruleAddr);
         CHKERR(eltp);
         *eltp = 0;
-        if (secAttach() < 0)
-        {
-                writeMemo("[?] sec_get_bspPcbRule can't find ION security.");
-                return 0;
-        }
+	if (secdb == NULL)	/*	No security database declared.	*/
+	{
+		return 0;
+	}
 
-        sdr_begin_xn(sdr);
+        CHKERR(sdr_begin_xn(sdr));
         for (elt = sdr_list_first(sdr, secdb->bspPcbRules); elt;
                         elt = sdr_list_next(sdr, elt))
         {
@@ -1449,12 +1391,6 @@ int     sec_findBspPcbRule(char *secSrcEid, char *secDestEid, int BlockTypeNbr,
         CHKERR(ruleAddr);
         CHKERR(eltp);
         *eltp = 0;
-        if (secAttach() < 0)
-        {
-                writeMemo("[?] sec_findBspPcbRule can't find ION security.");
-                return -1;
-        }
-
         if ((filterEid(secSrcEid, secSrcEid) == 0)
         || (filterEid(secDestEid, secDestEid) == 0))
         {
@@ -1478,12 +1414,7 @@ int     sec_addBspPcbRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
         CHKERR(secDestEid);
         CHKERR(ciphersuiteName);
         CHKERR(keyName);
-        if (secAttach() < 0)
-        {
-                writeMemo("[?] sec_addBspPcbRule can't find ION security.");
-                return 0;
-        }
-        
+        CHKERR(secdb);
         if (strlen(ciphersuiteName) > 31)
         {
                 writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -1513,7 +1444,7 @@ int     sec_addBspPcbRule(char *secSrcEid, char *secDestEid, int blockTypeNbr,
 
         /*      Okay to add this rule to the database.                  */
 
-        sdr_begin_xn(sdr);
+        CHKERR(sdr_begin_xn(sdr));
         rule.securitySrcEid = sdr_string_create(sdr, secSrcEid);
         rule.securityDestEid = sdr_string_create(sdr, secDestEid);
         rule.blockTypeNbr = blockTypeNbr;
@@ -1552,12 +1483,6 @@ int     sec_updateBspPcbRule(char *secSrcEid, char *secDestEid,
         CHKERR(secDestEid);
         CHKERR(ciphersuiteName);
         CHKERR(keyName);
-        if (secAttach() < 0)
-        {
-                writeMemo("[?] sec_updatePcbRule can't find ION security.");
-                return 0;
-        }
-
         if (strlen(ciphersuiteName) > 31)
         {
                 writeMemoNote("[?] Invalid ciphersuiteName", ciphersuiteName);
@@ -1585,7 +1510,7 @@ int     sec_updateBspPcbRule(char *secSrcEid, char *secDestEid,
                 return 0;
         }
 
-        sdr_begin_xn(sdr);
+        CHKERR(sdr_begin_xn(sdr));
         sdr_stage(sdr, (char *) &rule, ruleObj, sizeof(BspPcbRule));
         istrcpy(rule.ciphersuiteName, ciphersuiteName,
                         sizeof rule.ciphersuiteName);
@@ -1610,12 +1535,6 @@ int     sec_removeBspPcbRule(char *secSrcEid, char *secDestEid,
 
         CHKERR(secSrcEid);
         CHKERR(secDestEid);
-        if (secAttach() < 0)
-        {
-                writeMemo("[?] sec_removeBspPcbRule can't find ION security.");
-                return 0;
-        }
-
         if ((filterEid(secSrcEid, secSrcEid) == 0)
         || (filterEid(secDestEid, secDestEid) == 0))
         {
@@ -1631,7 +1550,7 @@ int     sec_removeBspPcbRule(char *secSrcEid, char *secDestEid,
                 return 0;
         }
 
-        sdr_begin_xn(sdr);
+        CHKERR(sdr_begin_xn(sdr));
         sdr_list_delete(sdr, elt, NULL, NULL);
         GET_OBJ_POINTER(sdr, BspPcbRule, rule, ruleObj);
         sdr_free(sdr, rule->securitySrcEid);
