@@ -476,9 +476,15 @@ void	unlockResource(ResourceLock *rl)
 
 #endif				/*	end #ifdef _MULTITHREADED	*/
 
-#if (!defined (linux) && !defined (freebsd) && !defined (darwin) && !defined (RTEMS)) && !defined (mingw)
+#if (!defined (linux) && !defined (freebsd) && !defined (darwin) && !defined (RTEMS) && !defined (mingw))
 /*	These things are defined elsewhere for Linux-like op systems.	*/
 
+#ifdef solaris
+char	*system_error_msg()
+{
+	return strerror(errno);
+}
+#else
 extern int	sys_nerr;
 extern char	*sys_errlist[];
 
@@ -491,6 +497,7 @@ char	*system_error_msg()
 
 	return sys_errlist[errno];
 }
+#endif				/*	end #ifdef solaris		*/
 
 char	*getNameOfUser(char *buffer)
 {
@@ -654,6 +661,10 @@ int	reUseAddress(int fd)
 
 	result = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &i,
 			sizeof i);
+#if (defined (SO_REUSEPORT))
+	result += setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, (char *) &i,
+			sizeof i);
+#endif
 	if (result < 0)
 	{
 		putSysErrmsg("can't make socket address reusable", NULL);
@@ -734,6 +745,9 @@ int	reUseAddress(int fd)
 	int	i = 1;
 
 	result = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof i);
+#if (defined (SO_REUSEPORT))
+	result += setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, &i, sizeof i);
+#endif
 	if (result < 0)
 	{
 		putSysErrmsg("can't make socket address reusable", NULL);
@@ -1053,6 +1067,10 @@ int	reUseAddress(int fd)
 
 	result = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void *) &i,
 			sizeof i);
+#if (defined (SO_REUSEPORT))
+	result += setsockopt(fd, SOL_SOCKET, SO_REUSEPORT, (char *) &i,
+			sizeof i);
+#endif
 	if (result < 0)
 	{
 		putSysErrmsg("can't make socket address reusable", NULL);
@@ -1565,6 +1583,7 @@ int	_iEnd(const char *fileName, int lineNbr, const char *arg)
 {
 	_postErrmsg(fileName, lineNbr, "Assertion failed.", arg);
 	writeErrmsgMemos();
+	printStackTrace();
 	if (_coreFileNeeded(NULL))
 	{
 		sm_Abort();
@@ -1573,13 +1592,42 @@ int	_iEnd(const char *fileName, int lineNbr, const char *arg)
 	return 1;
 }
 
-void	encodeSdnv(Sdnv *sdnv, unsigned long val)
+void	printStackTrace()
 {
-	static unsigned long	sdnvMask = ((unsigned long) -1) / 128;
-	unsigned long		remnant;
-	int			i;
-	unsigned char		flag = 0;
-	unsigned char		*text;
+#if (defined(bionic) || defined(uClibc) || !(defined(linux)))
+	writeMemo("[?] No stack trace available on this platform.");
+#else
+#define	MAX_TRACE_DEPTH	100
+	void	*returnAddresses[MAX_TRACE_DEPTH];
+	size_t	stackFrameCount;
+	char	**functionNames;
+	int	i;
+
+	stackFrameCount = backtrace(returnAddresses, MAX_TRACE_DEPTH);
+	functionNames = backtrace_symbols(returnAddresses, stackFrameCount);
+	if (functionNames == NULL)
+	{
+		writeMemo("[!] Can't print backtrace function names.");
+		return;
+	}
+
+	writeMemo("[i] Current stack trace:");
+	for (i = 0; i < stackFrameCount; i++)
+	{
+		writeMemoNote("[i] ", functionNames[i]);
+	}
+
+	free(functionNames);
+#endif
+}
+
+void	encodeSdnv(Sdnv *sdnv, uvast val)
+{
+	static uvast	sdnvMask = ((uvast) -1) / 128;
+	uvast		remnant;
+	int		i;
+	unsigned char	flag = 0;
+	unsigned char	*text;
 
 	/*	Get length of SDNV text: one byte for each 7 bits of
 	 *	significant numeric value.  On each iteration of the
@@ -1619,7 +1667,7 @@ void	encodeSdnv(Sdnv *sdnv, unsigned long val)
 	}
 }
 
-int	decodeSdnv(unsigned long *val, unsigned char *sdnvTxt)
+int	decodeSdnv(uvast *val, unsigned char *sdnvTxt)
 {
 	int		sdnvLength = 0;
 	unsigned char	*cursor;
@@ -2114,6 +2162,10 @@ int	parseSocketSpec(char *socketSpec, unsigned short *portNbr,
 	char		*delimiter;
 	char		*hostname;
 	char		hostnameBuf[MAXHOSTNAMELEN + 1];
+	unsigned int	d1;
+	unsigned int	d2;
+	unsigned int	d3;
+	unsigned int	d4;
 	unsigned int	i4;
 
 	CHKERR(portNbr);
@@ -2148,6 +2200,16 @@ int	parseSocketSpec(char *socketSpec, unsigned short *portNbr,
 				getNameOfHost(hostnameBuf, sizeof hostnameBuf);
 				hostname = hostnameBuf;
 			}
+			else
+			{
+				if (strcmp(hostname, "localhost") != 0
+				&& sscanf(hostname, "%u.%u.%u.%u", &d1, &d2,
+						&d3, &d4) != 4)
+				{
+					writeMemoNote("[!] Warning: safer to \
+use a dotted string IP address than a possibly aliased host name", hostname);
+				}
+			}
 
 			i4 = getInternetAddress(hostname);
 			if (i4 < 1)	/*	Invalid hostname.	*/
@@ -2198,6 +2260,13 @@ int	parseSocketSpec(char *socketSpec, unsigned short *portNbr,
 }
 #endif
 
+void	printDottedString(unsigned int hostNbr, char *buffer)
+{
+	CHKVOID(buffer);
+	isprintf(buffer, 16, "%u.%u.%u.%u", (hostNbr >> 24) & 0xff,
+		(hostNbr >> 16) & 0xff, (hostNbr >> 8) & 0xff, hostNbr & 0xff);
+}
+
 /*	Portable implementation of a safe snprintf: always NULL-
  *	terminates the content of the string composition buffer.	*/
 
@@ -2327,22 +2396,24 @@ static void	snGetNumber(char **cursor, char *fmt, int *fmtLen, int *number)
 
 int	_isprintf(char *buffer, int bufSize, char *format, ...)
 {
-	va_list	args;
-	char	*cursor;
-	int	stringLength = 0;
-	int	printLength = 0;
-	char	fmt[SN_FMT_SIZE];
-	int	fmtLen;
-	int	minFieldLength;
-	int	precision;
-	char	scratchpad[64];
-	int	numLen;
-	int	fieldLength;
-	int	*ipval;
-	char	*sval;
-	int	ival;
-	double	dval;
-	void	*vpval;
+	va_list		args;
+	char		*cursor;
+	int		stringLength = 0;
+	int		printLength = 0;
+	char		fmt[SN_FMT_SIZE];
+	int		fmtLen;
+	int		minFieldLength;
+	int		precision;
+	char		scratchpad[64];
+	int		numLen;
+	int		fieldLength;
+	int		isLongLong;		/*	Boolean		*/
+	int		*ipval;
+	char		*sval;
+	int		ival;
+	long long	llval;
+	double		dval;
+	void		*vpval;
 
 	if (buffer == NULL || bufSize < 1)
 	{
@@ -2455,14 +2526,50 @@ int	_isprintf(char *buffer, int bufSize, char *format, ...)
 
 		/*	Copy the field's length modifier, if any.	*/
 
-		if ((*cursor) == 'h'
-		|| (*cursor) == 'l'
-		|| (*cursor) == 'L')
+		isLongLong = 0;
+		if ((*cursor) == 'h'		/*	Short.		*/
+		|| (*cursor) == 'L')		/*	Long double.	*/
 		{
 			fmt[fmtLen] = *cursor;
 			fmtLen++;
 			cursor++;
 		}
+		else
+		{
+			if ((*cursor) == 'l')	/*	Long...		*/
+			{
+				fmt[fmtLen] = *cursor;
+				fmtLen++;
+				cursor++;
+				if ((*cursor) == 'l')	/*	Vast.	*/
+				{
+					isLongLong = 1;
+					fmt[fmtLen] = *cursor;
+					fmtLen++;
+					cursor++;
+				}
+			}
+			else
+			{
+				if ((*cursor) == 'I'
+				&& (*(cursor + 1)) == '6'
+				&& (*(cursor + 2)) == '4')
+				{
+#ifdef mingw
+					isLongLong = 1;
+					fmt[fmtLen] = *cursor;
+					fmtLen++;
+					cursor++;
+					fmt[fmtLen] = *cursor;
+					fmtLen++;
+					cursor++;
+					fmt[fmtLen] = *cursor;
+					fmtLen++;
+					cursor++;
+#endif
+				}
+			}
+		}
 
 		/*	Handle a couple of weird conversion characters
 		 *	as applicable.					*/
@@ -2538,14 +2645,23 @@ int	_isprintf(char *buffer, int bufSize, char *format, ...)
 		switch (*cursor)
 		{
 		case 'd':
+		case 'u':
 		case 'i':
 		case 'o':
 		case 'x':
 		case 'X':
-		case 'u':
 		case 'c':
-			ival = va_arg(args, int);
-			sprintf(scratchpad, fmt, ival);
+			if (isLongLong)
+			{
+				llval = va_arg(args, long long);
+				sprintf(scratchpad, fmt, llval);
+			}
+			else
+			{
+				ival = va_arg(args, int);
+				sprintf(scratchpad, fmt, ival);
+			}
+
 			break;
 
 		case 'f':
