@@ -34,7 +34,7 @@ int	sm_GetUniqueKey()
 
 	takeIpcLock();
 	ipcUniqueKey++;
-	result = (int) ipcUniqueKey;
+	result = ipcUniqueKey;		/*	Truncates as necessary.	*/
 	giveIpcLock();
 	return result;
 }
@@ -441,7 +441,7 @@ sm_ShmAttach(int key, int size, char **shmPtr, int *id)
     /* create a new shared memory segment, or attach to an existing one */
 	if ((*id = shmget(key, size, IPC_CREAT | 0666)) == -1)
 	{
-		putSysErrmsg("Can't get shared memory segment", itoa(key));
+		putSysErrmsg("Can't get shared memory segment", itoa(size));
 		return -1;
 	}
 
@@ -459,7 +459,7 @@ sm_ShmAttach(int key, int size, char **shmPtr, int *id)
 	 *	calling sm_ShmAttach, to let shmat determine the
 	 *	attachment point for the memory segment.		*/
 
-	if ((long) (mem = shmat(*id, *shmPtr, 0)) == -1)
+	if ((mem = (char *) shmat(*id, *shmPtr, 0)) == ((char *) -1))
 	{
 		putSysErrmsg("Can't attach shared memory segment", itoa(key));
 		return -1;
@@ -616,7 +616,7 @@ static void	tagArgBuffers(int tid)
 	oK(_argBuffersAvbl(&avbl));
 }
 
-#endif			/*	End of #if defined (VXWORKS, RTEMS)	*/
+#endif		/*	End of #if defined (VXWORKS, RTEMS, bionic)	*/
 
 /****************** Semaphore services **********************************/
 
@@ -947,6 +947,35 @@ void	sm_SemUnend(sm_SemId i)
 	sem->ended = 0;
 }
 
+int	sm_SemUnwedge(sm_SemId i, int timeoutSeconds)
+{
+	SmSem	*semTbl = _semTbl();
+	SmSem	*sem;
+	int	ticks;
+
+	CHKERR(i >= 0);
+	CHKERR(i < nSemIds);
+	sem = semTbl + i;
+	if (timeoutSeconds < 1) timeoutSeconds = 1;
+	ticks = sysClkRateGet() * timeoutSeconds;
+	if (semTake(sem->id, ticks) == ERROR)
+	{
+		if (errno != S_objLib_OBJ_TIMEOUT)
+		{
+			putSysErrmsg("Can't unwedge semaphore", itoa(i));
+			return -1;
+		}
+	}
+
+	if (semGive(sem->id) == ERROR)
+	{
+		putSysErrmsg("Can't unwedge semaphore", itoa(i));
+		return -1;
+	}
+
+	return 0;
+}
+
 #endif			/*	End of #ifdef VXWORKS_SEMAPHORES	*/
 
 #ifdef MINGW_SEMAPHORES
@@ -1164,7 +1193,6 @@ int	sm_SemTake(sm_SemId i)
 	SemaphoreTable	*semTbl = _semTbl(0);
 	IciSemaphore	*sem;
 	HANDLE		semId;
-	int		result = 0;
 
 	CHKERR(i >= 0);
 	CHKERR(i < NUM_SEMAPHORES);
@@ -1178,7 +1206,7 @@ int	sm_SemTake(sm_SemId i)
 
 	oK(WaitForSingleObject(semId, INFINITE));
 	CloseHandle(semId);
-	return result;
+	return 0;
 }
 
 void	sm_SemGive(sm_SemId i)
@@ -1243,6 +1271,31 @@ void	sm_SemUnend(sm_SemId i)
 	sem->ended = 0;
 }
 
+int	sm_SemUnwedge(sm_SemId i, int timeoutSeconds)
+{
+	SemaphoreTable	*semTbl = _semTbl(0);
+	IciSemaphore	*sem;
+	HANDLE		semId;
+	DWORD		millisec;
+
+	CHKERR(i >= 0);
+	CHKERR(i < NUM_SEMAPHORES);
+	sem = semTbl->semaphores + i;
+	semId = getSemaphoreHandle(sem->key);
+	if (semId == NULL)
+	{
+		putSysErrmsg("Can't unwedge semaphore", itoa(i));
+		return -1;
+	}
+
+	if (timeoutSeconds < 1) timeoutSeconds = 1;
+	millisec = timeoutSeconds * 1000;
+	oK(WaitForSingleObject(semId, millisec));
+	oK(SetEvent(semId));
+	CloseHandle(semId);
+	return 0;
+}
+
 #endif			/*	End of #ifdef MINGW_SEMAPHORES		*/
 
 #ifdef POSIX1B_SEMAPHORES
@@ -1417,17 +1470,21 @@ int	sm_SemTake(sm_SemId i)
 {
 	SmSem	*semTbl = _semTbl();
 	SmSem	*sem = semTbl + i;
-	int	result;
 
 	CHKERR(i >= 0);
 	CHKERR(i < SEM_NSEMS_MAX);
-	result = sem_wait(sem->id);
-	if (result < 0)
+	while (sem_wait(sem->id) < 0)
 	{
+		if (errno == EINTR)
+		{
+			continue;
+		}
+
 		putSysErrmsg("Can't take semaphore", itoa(i));
+		return -1;
 	}
 
-	return result;
+	return 0;
 }
 
 void	sm_SemGive(sm_SemId i)
@@ -1481,6 +1538,44 @@ void	sm_SemUnend(sm_SemId i)
 	sem->ended = 0;
 }
 
+int	sm_SemUnwedge(sm_SemId i, int timeoutSeconds)
+{
+	SmSem		*semTbl = _semTbl();
+	SmSem		*sem = semTbl + i;
+	struct timespec	timeout;
+
+	CHKERR(i >= 0);
+	CHKERR(i < SEM_NSEMS_MAX);
+	if (timeoutSeconds < 1) timeoutSeconds = 1;
+	oK(clock_gettime(CLOCK_REALTIME, &timeout));
+	timeout.tv_sec += timeoutSeconds;
+	while (sem_timedwait(sem->id, &timeout) < 0)
+	{
+		switch (errno)
+		{
+		case EINTR:
+			continue;
+			
+		case ETIMEDOUT:
+			break;	/*	Out of switch.			*/
+
+		default:
+			putSysErrmsg("Can't unwedge semaphore", itoa(i));
+			return -1;
+		}
+
+		break;		/*	Out of loop.			*/
+	}
+
+	if (sem_post(sem->id) < 0)
+	{
+		putSysErrmsg("Can't unwedge semaphore", itoa(i));
+		return -1;
+	}
+
+	return 0;
+}
+
 #endif			/*	End of #ifdef POSIX1B_SEMAPHORES	*/
 
 #ifdef SVR4_SEMAPHORES
@@ -1497,7 +1592,6 @@ void	sm_SemUnend(sm_SemId i)
 typedef struct
 {
 	int		semid;
-	int		setSize;
 	int		createCount;
 	int		destroyCount;
 } IciSemaphoreSet;
@@ -1530,6 +1624,7 @@ static SemaphoreBase	*_sembase(int stop)
 	static int		sembaseId = 0;
 	int			semSetIdx;
 	IciSemaphoreSet		*semset;
+	int			i;
 
 	if (stop)
 	{
@@ -1540,7 +1635,7 @@ static SemaphoreBase	*_sembase(int stop)
 			{
 				semset = semaphoreBase->semSets + semSetIdx;
 				oK(semctl(semset->semid, 0, IPC_RMID, NULL));
-            semSetIdx++;
+            			semSetIdx++;
 			}
 
 			sm_ShmDestroy(sembaseId);
@@ -1565,6 +1660,10 @@ static SemaphoreBase	*_sembase(int stop)
 		default:		/*	New SemaphoreBase.	*/
 			semaphoreBase->semaphoresCount = 0;
 			semaphoreBase->currSemSet = 0;
+			for (i = 0; i < SEMMNI; i++)
+			{
+				semaphoreBase->semSets[i].semid = -1;
+			}
 
 			/*	Acquire initial semaphore set.		*/
 
@@ -1581,7 +1680,6 @@ static SemaphoreBase	*_sembase(int stop)
 				break;
 			}
 
-			semset->setSize = SEMMSL;
 			semset->createCount = 0;
 			semset->destroyCount = 0;
 		}
@@ -1703,6 +1801,13 @@ sm_SemId	sm_SemCreate(int key, int semType)
 	/*	No existing semaphore for this key; allocate new one
 	 *	from next semaphore in current semaphore set.		*/
 
+	if (!(i < SEMMNS))
+	{
+		giveIpcLock();
+		putErrmsg("Can't add any more semaphores; table full.", NULL);
+		return SM_SEM_NONE;
+	}
+
 	sembase->semaphores[i].key = key;
 	sembase->semaphores[i].ended = 0;
 	sembase->semaphores[i].semSetIdx = sembase->currSemSet;
@@ -1713,7 +1818,7 @@ sm_SemId	sm_SemCreate(int key, int semType)
 	/*	Now roll over to next semaphore set if necessary.	*/
 
 	semset->createCount++;
-	if (semset->createCount == semset->setSize)
+	if (semset->createCount == SEMMSL)
 	{
 		/*	Must acquire another semaphore set.		*/
 
@@ -1729,12 +1834,12 @@ sm_SemId	sm_SemCreate(int key, int semType)
 		 *	managing this semaphore set.			*/
 
 		semSetIdx = sembase->currSemSet;
-		while (semset->setSize != 0)
+		while (1)
 		{
 			semSetIdx++;
 			if (semSetIdx == SEMMNI)
 			{
-				semSetIdx = 0;
+				semSetIdx = 0;	/*	Roll over.	*/
 			}
 
 			if (semSetIdx == sembase->currSemSet)
@@ -1746,11 +1851,14 @@ manage the new one.", NULL);
 			}
 
 			semset = sembase->semSets + semSetIdx;
+			if (semset->semid == -1)
+			{
+				break;	/*	Found unused row.	*/
+			}
 		}
 
 		sembase->currSemSet = semSetIdx;
 		semset->semid = semid;
-		semset->setSize = SEMMSL;
 		semset->createCount = 0;
 		semset->destroyCount = 0;
 	}
@@ -1780,7 +1888,7 @@ void	sm_SemDelete(sm_SemId i)
 
 	semset = sembase->semSets + sem->semSetIdx;
 	semset->destroyCount++;
-	if (semset->destroyCount == semset->setSize)
+	if (semset->destroyCount == SEMMSL)
 	{
 		/*	All semaphores in this set have been deleted,
 		 *	so we can release the entire semaphore set
@@ -1792,8 +1900,7 @@ void	sm_SemDelete(sm_SemId i)
 					itoa(semset->semid));
 		}
 
-		semset->semid = 0;
-		semset->setSize = 0;
+		semset->semid = -1;
 		semset->createCount = 0;
 		semset->destroyCount = 0;
 	}
@@ -1806,7 +1913,6 @@ int	sm_SemTake(sm_SemId i)
 	SemaphoreBase	*sembase = _sembase(0);
 	IciSemaphore	*sem;
 	IciSemaphoreSet	*semset;
-	int		result;
 	struct sembuf	sem_op[2] = { {0,0,0}, {0,1,0} };
 
 	CHKERR(sembase);
@@ -1821,21 +1927,19 @@ int	sm_SemTake(sm_SemId i)
 
 	semset = sembase->semSets + sem->semSetIdx;
 	sem_op[0].sem_num = sem_op[1].sem_num = sem->semNbr;
-	while (1)
+	while (semop(semset->semid, sem_op, 2) < 0)
 	{
-		result = semop(semset->semid, sem_op, 2);
-		if (result)
+		if (errno == EINTR)
 		{
-			if (errno == EINTR)
-			{
-				continue;
-			}
-
+			/*Retry on Interruption by signal*/
+			continue;
+		} else {
 			putSysErrmsg("Can't take semaphore", itoa(i));
+			return -1;
 		}
-
-		return result;
 	}
+
+	return 0;
 }
 
 void	sm_SemGive(sm_SemId i)
@@ -1851,7 +1955,7 @@ void	sm_SemGive(sm_SemId i)
 	sem = sembase->semaphores + i;
 	if (sem->key == -1)	/*	semaphore deleted		*/
 	{
-		writeMemoNote("[?] Can't give deleted semaphore", itoa(i));
+		putErrmsg("Can't give deleted semaphore.", itoa(i));
 		return;
 	}
 
@@ -1861,7 +1965,7 @@ void	sm_SemGive(sm_SemId i)
 	{
 		if (errno != EAGAIN)
 		{
-			writeMemoNote("[?] Can't give semaphore", itoa(i));
+			putSysErrmsg("Can't give semaphore", itoa(i));
 		}
 	}
 }
@@ -1910,6 +2014,58 @@ void	sm_SemUnend(sm_SemId i)
 	sem->ended = 0;
 }
 
+static void	handleTimeout()
+{
+	return;
+}
+
+int	sm_SemUnwedge(sm_SemId i, int timeoutSeconds)
+{
+	SemaphoreBase	*sembase = _sembase(0);
+	IciSemaphore	*sem;
+	IciSemaphoreSet	*semset;
+	struct sembuf	sem_op[3] = { {0,0,0}, {0,1,0}, {0,-1,IPC_NOWAIT} };
+
+	CHKERR(sembase);
+	CHKERR(i >= 0);
+	CHKERR(i < sembase->semaphoresCount);
+	sem = sembase->semaphores + i;
+	if (sem->key == -1)	/*	semaphore deleted		*/
+	{
+		putErrmsg("Can't unwedge deleted semaphore.", itoa(i));
+		return -1;
+	}
+
+	semset = sembase->semSets + sem->semSetIdx;
+	sem_op[0].sem_num = sem_op[1].sem_num = sem_op[2].sem_num = sem->semNbr;
+	if (timeoutSeconds < 1) timeoutSeconds = 1;
+	isignal(SIGALRM, handleTimeout);
+	oK(alarm(timeoutSeconds));
+	if (semop(semset->semid, sem_op, 2) < 0)
+	{
+		if (errno != EINTR)
+		{
+			putSysErrmsg("Can't take semaphore", itoa(i));
+			return -1;
+		}
+		/*Intentionally don't retry if EINTR... That means the
+		 *alarm we just set went off... We're going to proceed anyway.*/
+	}
+
+	oK(alarm(0));
+	isignal(SIGALRM, SIG_DFL);
+	if (semop(semset->semid, sem_op + 2, 1) < 0)
+	{
+		if (errno != EAGAIN)
+		{
+			putSysErrmsg("Can't give semaphore", itoa(i));
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
 #endif			/*	End of #ifdef SVR4_SEMAPHORES		*/
 
 /************************* Symbol table services  *****************************/
@@ -2069,7 +2225,65 @@ void	sm_Abort()
 
 #endif			/*	End of #ifdef VXWORKS			*/
 
-#if (defined(RTEMS) || defined(bionic))
+#if defined (bionic) || defined (uClibc)
+
+typedef struct
+{
+	void	*(*function)(void *);
+	void	*arg;
+} IonPthreadParm;
+
+static void	posixTaskExit(int sig)
+{
+	pthread_exit(0);
+}
+
+static void	sm_ArmPthread()
+{
+	struct sigaction	actions;
+
+	memset((char *) &actions, 0, sizeof actions);
+	sigemptyset(&actions.sa_mask);
+	actions.sa_flags = 0;
+	actions.sa_handler = posixTaskExit;
+	oK(sigaction(SIGUSR2, &actions, NULL));
+}
+
+void	sm_EndPthread(pthread_t threadId)
+{
+	/*	NOTE that this is NOT a faithful implementation of
+	 *	pthread_cancel(); there is no support for deferred
+	 *	thread cancellation in Bionic (the Android subset
+	 *	of Linux).  It's just a code simplification, solely
+	 *	for the express, limited purpose of shutting down a
+	 *	task immediately, under the highly constrained
+	 *	circumstances defined by sm_TaskSpawn, sm_TaskDelete,
+	 *	and sm_Abort, below.					*/
+
+	oK(pthread_kill(threadId, SIGUSR2));
+}
+
+static void	*posixTaskEntrance(void *arg)
+{
+	IonPthreadParm	*parm = (IonPthreadParm *) arg;
+
+	sm_ArmPthread();
+	return (parm->function)(parm->arg);
+}
+
+int	sm_BeginPthread(pthread_t *threadId, const pthread_attr_t *attr,
+		void *(*function)(void *), void *arg)
+{
+	IonPthreadParm	parm;
+
+	parm.function = function;
+	parm.arg = arg;
+	return pthread_create(threadId, attr, posixTaskEntrance, &parm);
+}
+
+#endif			/*	End of #if defined bionic || uClibc	*/
+
+#if defined (RTEMS) || defined (bionic)
 
 /*	Note: the RTEMS API is UNIX-like except that it omits all SVR4
  *	features.  RTEMS uses POSIX semaphores, and its shared-memory
@@ -2355,27 +2569,6 @@ typedef struct
 	int	arg10;
 } SpawnParms;
 
-#ifdef bionic
-static void	posixTaskExit(int sig)
-{
-	pthread_exit(0);
-}
-
-void	pthread_cancel(pthread_t threadId)
-{
-	/*	NOTE that this is NOT a faithful implementation of
-	 *	pthread_cancel(); there is no support for deferred
-	 *	thread cancellation in Bionic (the Android subset
-	 *	of Linux).  It's just a code simplification, solely
-	 *	for the express, limited purpose of shutting down a
-	 *	task immediately, under the highly constrained
-	 *	circumstances defined by sm_TaskSpawn, sm_TaskDelete,
-	 *	and sm_Abort, below.					*/
-
-	oK(pthread_kill(threadId, SIGUSR2));
-}
-#endif
-
 static void	*posixDriverThread(void *parm)
 {
 	SpawnParms	parms;
@@ -2388,16 +2581,10 @@ static void	*posixDriverThread(void *parm)
 
 	memset((char *) parm, 0, sizeof(SpawnParms));
 
-#ifdef bionic
-	/*	Set up SIGUSR2 handler to enable shutdown.		*/
+#if defined (bionic)
+	/*	Set up SIGUSR2 handler to enable clean task shutdown.	*/
 
-	struct sigaction	actions;
-
-	memset((char *) &actions, 0, sizeof actions);
-	sigemptyset(&actions.sa_mask);
-	actions.sa_flags = 0;
-	actions.sa_handler = posixTaskExit;
-	oK(sigaction(SIGUSR2, &actions, NULL));
+	sm_ArmPthread();
 #endif
 	/*	Run main function of thread.				*/
 
@@ -2477,7 +2664,7 @@ private symbol table; must be added to mysymtab.c.", name);
 	{
 		if (pthread_kill(threadId, SIGTERM) == 0)
 		{
-			oK(pthread_cancel(threadId));
+			oK(pthread_end(threadId));
 		}
 
 		return -1;
@@ -2516,7 +2703,7 @@ void	sm_TaskDelete(int taskId)
 
 	if (pthread_kill(threadId, SIGTERM) == 0)
 	{
-		oK(pthread_cancel(threadId));
+		oK(pthread_end(threadId));
 	}
 
 	oK(_posixTasks(&taskId, NULL, NULL));
@@ -2534,7 +2721,7 @@ void	sm_Abort()
 		threadId = pthread_self();
 		if (pthread_kill(threadId, SIGTERM) == 0)
 		{
-			oK(pthread_cancel(threadId));
+			oK(pthread_end(threadId));
 		}
 
 		return;
@@ -2790,7 +2977,8 @@ void	sm_Wakeup(DWORD processId)
 }
 #endif			/*	End of #ifdef mingw			*/
 
-#if (!defined(VXWORKS) && !defined(RTEMS) && !defined(mingw) && !defined(bionic))
+#if defined (VXWORKS) || defined (RTEMS) || defined (mingw) || defined (bionic)
+#else
 
 	/* ---- IPC services access control (Unix) -------------------- */
 
@@ -2870,6 +3058,20 @@ void	sm_TaskYield()
 	sched_yield();
 }
 
+static void	closeAllFileDescriptors()
+{
+	struct rlimit	limit;
+	int		i;
+
+	oK(getrlimit(RLIMIT_NOFILE, &limit));
+	for (i = 3; i < limit.rlim_cur; i++)
+	{
+		oK(close(i));
+	}
+
+	writeMemo("");	/*	Tell logger that log file is closed.	*/
+}
+
 int	sm_TaskSpawn(char *name, char *arg1, char *arg2, char *arg3,
 		char *arg4, char *arg5, char *arg6, char *arg7, char *arg8,
 		char *arg9, char *arg10, int priority, int stackSize)
@@ -2884,6 +3086,7 @@ int	sm_TaskSpawn(char *name, char *arg1, char *arg2, char *arg3,
 		return -1;
 
 	case 0:			/*	This is the child process.	*/
+		closeAllFileDescriptors();
 		execlp(name, name, arg1, arg2, arg3, arg4, arg5, arg6,
 				arg7, arg8, arg9, arg10, NULL);
 
@@ -2920,7 +3123,7 @@ void	sm_Abort()
 	abort();
 }
 
-#endif		/*	End of #if !defined (VXWORKS, RTEMS, mingw)	*/
+#endif		/*	End of #ifdef (VXWORKS, RTEMS, mingw, bionic)	*/
 
 /******************* platform-independent functions ***********************/
 
@@ -2940,138 +3143,6 @@ void	sm_ConfigurePthread(pthread_attr_t *attr, size_t stackSize)
 	}
 }
 
-typedef struct
-{
-	sm_SemId	semid;
-	int		unwedged;
-	pthread_cond_t	*cv;
-} UnwedgeParms;
-
-static void	*checkSemaphore(void *parm)
-{
-	UnwedgeParms	*parms = (UnwedgeParms *) parm;
-
-	sm_SemTake(parms->semid);
-
-	/*	If semid is wedged, hang until parent thread times
-	 *	out and gives it.
-	 *
-	 *	When have successfully taken the semaphore (one way
-	 *	or another), give it up immediately.			*/
-
-	sm_SemGive(parms->semid);
-	parms->unwedged = 1;
-
-	/*	If semid was not wedged, then the parent thread is
-	 *	still waiting on the condition variable, in which
-	 *	case we need to signal it right away.  In any case,
-	 *	no harm in signaling it.				*/
-
-	pthread_cond_signal(parms->cv);
-	return NULL;
-}
-
-int	sm_SemUnwedge(sm_SemId semid, int interval)
-{
-	struct timeval	workTime;
-	struct timespec	deadline;
-	pthread_mutex_t	mutex;
-	pthread_cond_t	cv;
-	UnwedgeParms	parms;
-	int		result = 0;	/*	Semaphore not wedged.	*/
-	pthread_attr_t	attr;
-	pthread_t	unwedgeThread;
-
-	CHKERR(interval > 0);
-	if (sm_ipc_init())	/*	Shouldn't be needed, but okay.	*/
-	{
-		putErrmsg("Can't initialize IPC.", NULL);
-		return -1;
-	}
-
-	parms.semid = semid;
-	parms.unwedged = 0;
-	memset((char *) &mutex, 0, sizeof mutex);
-	if (pthread_mutex_init(&mutex, NULL))
-	{
-		putSysErrmsg("Can't initialize mutex", NULL);
-		return -1;
-	}
-
-	memset((char *) &cv, 0, sizeof cv);
-	if (pthread_cond_init(&cv, NULL))
-	{
-		oK(pthread_mutex_destroy(&mutex));
-		putSysErrmsg("Can't initialize condition variable", NULL);
-		return -1;
-	}
-
-	parms.cv = &cv;
-	getCurrentTime(&workTime);
-	deadline.tv_sec = workTime.tv_sec + interval;
-	deadline.tv_nsec = 0;
-
-	/*	Spawn a separate thread that hangs on the semaphore
-	 *	if it is wedged.					*/
-
-	sm_ConfigurePthread(&attr, 0);
-	errno = pthread_create(&unwedgeThread, &attr, checkSemaphore, &parms);
-	if (errno)
-	{
-		oK(pthread_mutex_destroy(&mutex));
-		oK(pthread_cond_destroy(&cv));
-		putSysErrmsg("Can't create unwedge thread", NULL);
-		return -1;
-	}
-
-	/*	At this point the child might already have taken and
-	 *	released the semaphore and terminated, in which case
-	 *	we want NOT to wait for a signal from it.		*/
-
-	if (parms.unwedged == 0)	/*	Child not ended yet.	*/
-	{
-	/*	Wait for all-OK signal from child; if none, give the
-	 *	semaphore.  Other tasks may already be hanging on this
-	 *	semaphore, so giving it may or may not enable the
-	 *	child thread to terminate immediately.  However, we
-	 *	expect that the other task(s) waiting on this semaphore
-	 *	will give it promptly after taking it, so that the
-	 *	child thread itself will eventually be able to take
-	 *	the semaphore, give it, and terminate cleanly.		*/
-
-		oK(pthread_mutex_lock(&mutex));
-		result = pthread_cond_timedwait(&cv, &mutex, &deadline);
-		oK(pthread_mutex_unlock(&mutex));
-		if (result)	/*	NOT signaled by child thread.	*/
-		{
-			if (result != ETIMEDOUT)
-			{
-				errno = result;
-				oK(pthread_mutex_destroy(&mutex));
-				oK(pthread_cond_destroy(&cv));
-				putSysErrmsg("pthread_cond_timedwait failed",
-						NULL);
-				return -1;
-			}
-
-			/*	Timeout: child stuck, semaphore wedged.	*/
-
-			sm_SemGive(semid);
-			result = 1;
-		}
-	}
-
-	/*	Giving the semaphore enables the unwedge thread to
-	 *	take it and thereupon give it back and return.  This
-	 *	enables the unwedge thread to terminate cleanly so
-	 *	that pthread_join() completes.				*/
-
-	oK(pthread_join(unwedgeThread, NULL));
-	oK(pthread_mutex_destroy(&mutex));
-	oK(pthread_cond_destroy(&cv));
-	return result;
-}
-
 int	pseudoshell(char *commandLine)
 {
 	int	length;
