@@ -268,21 +268,18 @@ Address	sdr_hash_entry_value(Sdr sdrv, Object hash, Object entry)
 	Object	kvpairAddr;
 	KvPair	kvpair;
 
-	CHKERR(sdrv);
+	CHKERR(sdrFetchSafe(sdrv));
 	CHKERR(entry);
-	CHKERR(takeSdr(sdrv->sdr) == 0);
 	keyLength = sdr_table_user_data(sdrv, hash);
 	kvpairLength = sizeof(Address) + keyLength;
 	kvpairAddr = sdr_list_data(sdrv, entry);
 	sdr_read(sdrv, (char *) &kvpair, kvpairAddr, kvpairLength);
-	releaseSdr(sdrv->sdr);
 	return kvpair.value;
 }
 
 int	sdr_hash_retrieve(Sdr sdrv, Object hash, char *key, Address *value,
 		Object *entry)
 {
-	SdrState	*sdr;
 	int		keyLength;
 	int		kvpairLength;
 	int		rowSize;
@@ -300,12 +297,10 @@ int	sdr_hash_retrieve(Sdr sdrv, Object hash, char *key, Address *value,
 		*entry = 0;	/*	Default result.			*/
 	}
 
-	CHKERR(sdrv);
+	CHKERR(sdrFetchSafe(sdrv));
 	CHKERR(hash);
 	CHKERR(key);
 	CHKERR(value);
-	sdr = sdrv->sdr;
-	CHKERR(takeSdr(sdr) == 0);
 	keyLength = sdr_table_user_data(sdrv, hash);
 	kvpairLength = sizeof(Address) + keyLength;
 	sdr_table_dimensions(sdrv, hash, &rowSize, &rowCount);
@@ -334,11 +329,9 @@ int	sdr_hash_retrieve(Sdr sdrv, Object hash, char *key, Address *value,
 			*entry = elt;
 		}
 
-		releaseSdr(sdr);
 		return 1;	/*	Got it.				*/
 	}
 
-	releaseSdr(sdr);
 	return 0;		/*	Unable to retrieve value.	*/
 }
 
@@ -364,11 +357,9 @@ int	sdr_hash_count(Sdr sdrv, Object hash)
 	return count;
 }
 
-
-int sdr_hash_foreach(Sdr sdrv, Object hash,
-		sdr_hash_callback callback, void *args)
+int	sdr_hash_foreach(Sdr sdrv, Object hash, sdr_hash_callback callback,
+		void *args)
 {
-	SdrState	*sdr;
 	int		keyLength;
 	int		kvpairLength;
 	int		rowSize;
@@ -376,42 +367,45 @@ int sdr_hash_foreach(Sdr sdrv, Object hash,
 	int		rowNbr;
 	Address		rowAddr;
 	Object		listAddr;
-	Object	elt;
-	Object	kvpairAddr;
+	Object		elt;
+	Object		kvpairAddr;
 	KvPair		kvpair;
 
-	CHKERR(sdrv);
+	CHKERR(sdrFetchSafe(sdrv));
 	CHKERR(hash);
 	CHKERR(callback);
 	//Passing NULL args is OK (passed through to callback)
-	sdr = sdrv->sdr;
-	takeSdr(sdr);
 	keyLength = sdr_table_user_data(sdrv, hash);
 	kvpairLength = sizeof(Address) + keyLength;
 	sdr_table_dimensions(sdrv, hash, &rowSize, &rowCount);
 
-	/* Iterate over each row/bucket, loading the sdrlist of members of each. */
+	/*	Iterate over each row/bucket, loading the sdrlist
+	 *	of members of each.					*/
+
 	for (rowNbr = 0; rowNbr < rowCount; rowNbr++)
 	{
 		rowAddr = sdr_table_row(sdrv, hash, rowNbr);
 		sdr_read(sdrv, (char *) &listAddr, rowAddr, sizeof(Object));
 
-		/* Iterate over each member of this bucket. */
+		/*	Iterate over each member of this bucket.	*/
+
 		for (elt = sdr_list_first(sdrv, listAddr); elt;
 				elt = sdr_list_next(sdrv, elt))
 		{
 			kvpairAddr = sdr_list_data(sdrv, elt);
-			sdr_read(sdrv, (char *) &kvpair, kvpairAddr, kvpairLength);
+			sdr_read(sdrv, (char *) &kvpair, kvpairAddr,
+					kvpairLength);
+
+			/*	Call the callback passed to us with
+			 *	the key, value pair.			*/
 
-			/* Call the callback passed to us with the key, value pair. */
 			callback(sdrv, hash, kvpair.key, kvpair.value, args);
 		}
 	}
-	releaseSdr(sdr);
+
 	return 0;
 }
 
-
 int	Sdr_hash_revise(char *file, int line, Sdr sdrv, Object hash, char *key,
 		Address value)
 {
