@@ -7,6 +7,7 @@
 /*	All rights reserved.						*/
 /*	Author: Scott Burleigh, Jet Propulsion Laboratory		*/
 
+#include "zco.h"
 #include "rfx.h"
 
 static time_t	_referenceTime(time_t *newValue)
@@ -21,6 +22,24 @@ static time_t	_referenceTime(time_t *newValue)
 	return reftime;
 }
 
+static int	_forecastNeeded(int parm)
+{
+	static int	needed = 0;
+	int		result;
+
+	result = needed;
+	if (parm)	/*	Signaling that forecast is needed.	*/
+	{
+		needed = 1;
+	}
+	else		/*	Signaling intent to forecast as nec.	*/
+	{
+		needed = 0;	/*	Forecasting resets flag value.	*/
+	}
+
+	return result;
+}
+
 static int	_echo(int *newValue)
 {
 	static int	state = 0;
@@ -99,9 +118,11 @@ relative times (+ss) are computed.");
 	PUTS("\tm\tManage ION database: clock, space occupancy");
 	PUTS("\t   m utcdelta <local clock time minus UTC, in seconds>");
 	PUTS("\t   m clockerr <new known maximum clock error, in seconds>");
+	PUTS("\t   m clocksync [ { 0 | 1 } ]");
 	PUTS("\t   m production <new planned production rate, in bytes/sec>");
 	PUTS("\t   m consumption <new planned consumption rate, in bytes/sec>");
-	PUTS("\t   m occupancy <new occupancy limit value, in bytes>");
+	PUTS("\t   m occupancy <new ZCO heap occupancy limit, in MB; -1 means \
+\"unchanged\"> [<new ZCO file occupancy limit, in MB>]");
 	PUTS("\t   m horizon { 0 | <end time for congestion forecasts> }");
 	PUTS("\t   m alarm '<congestion alarm script>'");
 	PUTS("\t   m usage");
@@ -140,7 +161,7 @@ static void	initializeNode(int tokenCount, char **tokens)
 		return;
 	}
 
-	if (ionInitialize(&parms, strtol(ownNodeNbrString, NULL, 0)) < 0)
+	if (ionInitialize(&parms, strtouvast(ownNodeNbrString)) < 0)
 	{
 		putErrmsg("ionadmin can't initialize ION.", NULL);
 	}
@@ -151,9 +172,9 @@ static void	executeAdd(int tokenCount, char **tokens)
 	time_t		refTime;
 	time_t		fromTime;
 	time_t		toTime;
-	unsigned long	fromNodeNbr;
-	unsigned long	toNodeNbr;
-	unsigned long	xmitRate;
+	uvast		fromNodeNbr;
+	uvast		toNodeNbr;
+	unsigned int	xmitRate;
 	unsigned int	owlt;
 
 	if (tokenCount < 2)
@@ -177,13 +198,14 @@ static void	executeAdd(int tokenCount, char **tokens)
 		return;
 	}
 
-	fromNodeNbr = strtol(tokens[4], NULL, 0);
-	toNodeNbr = strtol(tokens[5], NULL, 0);
+	fromNodeNbr = strtouvast(tokens[4]);
+	toNodeNbr = strtouvast(tokens[5]);
 	if (strcmp(tokens[1], "contact") == 0)
 	{
 		xmitRate = strtol(tokens[6], NULL, 0);
 		oK(rfx_insert_contact(fromTime, toTime, fromNodeNbr,
 				toNodeNbr, xmitRate));
+		oK(_forecastNeeded(1));
 		return;
 	}
 
@@ -200,10 +222,10 @@ static void	executeAdd(int tokenCount, char **tokens)
 
 static void	executeDelete(int tokenCount, char **tokens)
 {
-	time_t		refTime;
-	time_t		timestamp;
-	unsigned long	fromNodeNbr;
-	unsigned long	toNodeNbr;
+	time_t	refTime;
+	time_t	timestamp;
+	uvast	fromNodeNbr;
+	uvast	toNodeNbr;
 
 	if (tokenCount < 2)
 	{
@@ -232,11 +254,12 @@ static void	executeDelete(int tokenCount, char **tokens)
 		}
 	}
 
-	fromNodeNbr = strtol(tokens[3], NULL, 0);
-	toNodeNbr = strtol(tokens[4], NULL, 0);
+	fromNodeNbr = strtouvast(tokens[3]);
+	toNodeNbr = strtouvast(tokens[4]);
 	if (strcmp(tokens[1], "contact") == 0)
 	{
 		oK(rfx_remove_contact(timestamp, fromNodeNbr, toNodeNbr));
+		oK(_forecastNeeded(1));
 		return;
 	}
 
@@ -251,12 +274,13 @@ static void	executeDelete(int tokenCount, char **tokens)
 
 static void	executeInfo(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
 	IonVdb		*vdb = getIonVdb();
 	time_t		refTime;
 	time_t		timestamp;
-	unsigned long	fromNode;
-	unsigned long	toNode;
+	uvast		fromNode;
+	uvast		toNode;
 	IonCXref	arg1;
 	PsmAddress	elt;
 	PsmAddress	addr;
@@ -278,14 +302,15 @@ static void	executeInfo(int tokenCount, char **tokens)
 
 	refTime = _referenceTime(NULL);
 	timestamp = readTimestampUTC(tokens[2], refTime);
-	fromNode = strtol(tokens[3], NULL, 0);
-	toNode = strtol(tokens[4], NULL, 0);
+	fromNode = strtouvast(tokens[3]);
+	toNode = strtouvast(tokens[4]);
 	if (strcmp(tokens[1], "contact") == 0)
 	{
 		memset((char *) &arg1, 0, sizeof(IonCXref));
 		arg1.fromNode = fromNode;
 		arg1.toNode = toNode;
 		arg1.fromTime = timestamp;
+		CHKVOID(sdr_begin_xn(sdr));
 		elt = sm_rbt_search(ionwm, vdb->contactIndex,
 				rfx_order_contacts, &arg1, &nextElt);
 		if (elt)
@@ -293,10 +318,13 @@ static void	executeInfo(int tokenCount, char **tokens)
 			addr = sm_rbt_data(ionwm, elt);
 			oK(rfx_print_contact(addr, buffer));
 			printText(buffer);
-			return;
+		}
+		else
+		{
+			printText("Contact not found in database.");
 		}
 
-		printText("Contact not found in database.");
+		sdr_exit_xn(sdr);
 		return;
 	}
 
@@ -306,6 +334,7 @@ static void	executeInfo(int tokenCount, char **tokens)
 		arg2.fromNode = fromNode;
 		arg2.toNode = toNode;
 		arg2.fromTime = timestamp;
+		CHKVOID(sdr_begin_xn(sdr));
 		elt = sm_rbt_search(ionwm, vdb->rangeIndex,
 				rfx_order_ranges, &arg2, &nextElt);
 		if (elt)
@@ -313,10 +342,13 @@ static void	executeInfo(int tokenCount, char **tokens)
 			addr = sm_rbt_data(ionwm, elt);
 			oK(rfx_print_range(addr, buffer));
 			printText(buffer);
-			return;
+		}
+		else
+		{
+			printText("Range not found in database.");
 		}
 
-		printText("Range not found in database.");
+		sdr_exit_xn(sdr);
 		return;
 	}
 
@@ -325,6 +357,7 @@ static void	executeInfo(int tokenCount, char **tokens)
 
 static void	executeList(int tokenCount, char **tokens)
 {
+	Sdr		sdr = getIonsdr();
 	PsmPartition	ionwm = getIonwm();
 	IonVdb		*vdb = getIonVdb();
 	PsmAddress	elt;
@@ -339,6 +372,7 @@ static void	executeList(int tokenCount, char **tokens)
 
 	if (strcmp(tokens[1], "contact") == 0)
 	{
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sm_rbt_first(ionwm, vdb->contactIndex); elt;
 				elt = sm_rbt_next(ionwm, elt))
 		{
@@ -347,11 +381,13 @@ static void	executeList(int tokenCount, char **tokens)
 			printText(buffer);
 		}
 
+		sdr_exit_xn(sdr);
 		return;
 	}
 
 	if (strcmp(tokens[1], "range") == 0)
 	{
+		CHKVOID(sdr_begin_xn(sdr));
 		for (elt = sm_rbt_first(ionwm, vdb->rangeIndex); elt;
 				elt = sm_rbt_next(ionwm, elt))
 		{
@@ -360,6 +396,7 @@ static void	executeList(int tokenCount, char **tokens)
 			printText(buffer);
 		}
 
+		sdr_exit_xn(sdr);
 		return;
 	}
 
@@ -370,7 +407,7 @@ static void	manageUtcDelta(int tokenCount, char **tokens)
 {
 	int	newDelta;
 
-	if (tokenCount!= 3)
+	if (tokenCount != 3)
 	{
 		SYNTAX_ERROR;
 		return;
@@ -387,7 +424,7 @@ static void	manageClockError(int tokenCount, char **tokens)
 	IonDB	iondb;
 	int	newMaxClockError;
 
-	if (tokenCount!= 3)
+	if (tokenCount != 3)
 	{
 		SYNTAX_ERROR;
 		return;
@@ -400,7 +437,7 @@ static void	manageClockError(int tokenCount, char **tokens)
 		return;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 	iondb.maxClockError = newMaxClockError;
 	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
@@ -410,6 +447,40 @@ static void	manageClockError(int tokenCount, char **tokens)
 	}
 }
 
+static void	manageClockSync(int tokenCount, char **tokens)
+{
+	Sdr	sdr;
+	Object	iondbObj;
+	IonDB	iondb;
+	int	newSyncVal;
+	char	buffer[128];
+
+	if (tokenCount < 2 || tokenCount > 3)
+	{
+		SYNTAX_ERROR;
+		return;
+	}
+
+	if (tokenCount == 3)
+	{
+		newSyncVal = atoi(tokens[2]);
+		sdr = getIonsdr();
+		iondbObj = getIonDbObject();
+		CHKVOID(sdr_begin_xn(sdr));
+		sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
+		iondb.clockIsSynchronized = (!(newSyncVal == 0));
+		sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
+		if (sdr_end_xn(sdr) < 0)
+		{
+			putErrmsg("Can't change clock sync.", NULL);
+		}
+	}
+
+	isprintf(buffer, sizeof buffer, "clock sync = %d",
+			ionClockIsSynchronized());
+	printText(buffer);
+}
+
 static void	manageProduction(int tokenCount, char **tokens)
 {
 	Sdr	sdr = getIonsdr();
@@ -426,12 +497,10 @@ static void	manageProduction(int tokenCount, char **tokens)
 	newRate = atoi(tokens[2]);
 	if (newRate < 0)
 	{
-		putErrmsg("Planned bundle production rate can't be negative.",
-				itoa(newRate));
-		return;
+		newRate = -1;			/*	Not metered.	*/
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 	iondb.productionRate = newRate;
 	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
@@ -439,6 +508,8 @@ static void	manageProduction(int tokenCount, char **tokens)
 	{
 		putErrmsg("Can't change bundle production rate.", NULL);
 	}
+
+	oK(_forecastNeeded(1));
 }
 
 static void	manageConsumption(int tokenCount, char **tokens)
@@ -457,12 +528,10 @@ static void	manageConsumption(int tokenCount, char **tokens)
 	newRate = atoi(tokens[2]);
 	if (newRate < 0)
 	{
-		putErrmsg("Planned bundle consumption rate can't be negative.",
-				itoa(newRate));
-		return;
+		newRate = -1;			/*	Not metered.	*/
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 	iondb.consumptionRate = newRate;
 	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
@@ -470,6 +539,8 @@ static void	manageConsumption(int tokenCount, char **tokens)
 	{
 		putErrmsg("Can't change bundle consumption rate.", NULL);
 	}
+
+	oK(_forecastNeeded(1));
 }
 
 static void	manageOccupancy(int tokenCount, char **tokens)
@@ -477,31 +548,83 @@ static void	manageOccupancy(int tokenCount, char **tokens)
 	Sdr	sdr = getIonsdr();
 	Object	iondbObj = getIonDbObject();
 	IonDB	iondb;
-	int	newLimit;
+	vast	newFileLimit = -1;	/*	-1 = "unchanged"	*/
+	vast	newHeapLimit = -1;	/*	-1 = "unchanged"	*/
+	vast	fileLimit;
+	vast	maxHeapLimit;
+	vast	heapLimit;
 
-	if (tokenCount != 3)
+	switch (tokenCount)
 	{
+	case 4:
+		newFileLimit = strtovast(tokens[3]);
+
+		/*	Intentional fall-through to next case.		*/
+
+	case 3:
+		newHeapLimit = strtovast(tokens[2]);
+		break;
+
+	default:
 		SYNTAX_ERROR;
 		return;
 	}
 
-	newLimit = atoi(tokens[2]);
-	if (newLimit <= 0)
+	if (newFileLimit < -1)
 	{
-		putErrmsg("Bundle storage occupancy limit must be greater \
-than zero.", itoa(newLimit));
+		writeMemo("[?] ZCO file occupancy limit can't be negative.");
 		return;
 	}
 
-	sdr_begin_xn(sdr);
+	if (newHeapLimit < -1)
+	{
+		writeMemo("[?] ZCO heap occupancy limit can't be negative.");
+		return;
+	}
+
+	CHKVOID(sdr_begin_xn(sdr));
+	if (newFileLimit != -1)	/*	Overriding current value.	*/
+	{
+		fileLimit = newFileLimit * 1000000;
+
+		/*	Convert from MB to bytes.			*/
+
+		zco_set_max_file_occupancy(sdr, fileLimit);
+		writeMemo("[i] ZCO max file space changed.");
+	}
+
+	if (newHeapLimit != -1)	/*	Overriding the default.		*/
+	{
+		maxHeapLimit = (sdr_heap_size(sdr) / 100)
+				* (100 - ION_SEQUESTERED);
+		if (newHeapLimit > (maxHeapLimit / 1000000))
+		{
+			writeMemo("[i] New ZCO heap limit invalid!");
+		}
+		else
+		{
+			heapLimit = newHeapLimit * 1000000;
+
+			/*	Convert from MB to bytes.		*/
+
+			zco_set_max_heap_occupancy(sdr, heapLimit);
+			writeMemo("[i] ZCO max heap changed.");
+		}
+	}
+
+	/*	Revise occupancy ceiling and reserve as needed.		*/
+
+	fileLimit = zco_get_max_file_occupancy(sdr);
+	heapLimit = zco_get_max_heap_occupancy(sdr);
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
-	iondb.occupancyCeiling = newLimit;
+	iondb.occupancyCeiling = fileLimit + heapLimit;
 	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
 	if (sdr_end_xn(sdr) < 0)
 	{
-		putErrmsg("Can't change bundle storage occupancy limit.",
-				NULL);
+		putErrmsg("Can't change bundle storage occupancy limit.", NULL);
 	}
+
+	oK(_forecastNeeded(1));
 }
 
 static void	manageHorizon(int tokenCount, char **tokens)
@@ -530,7 +653,7 @@ static void	manageHorizon(int tokenCount, char **tokens)
 		horizon = readTimestampUTC(horizonString, refTime);
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 	iondb.horizon = horizon;
 	sdr_write(sdr, iondbObj, (char *) &iondb, sizeof(IonDB));
@@ -538,6 +661,8 @@ static void	manageHorizon(int tokenCount, char **tokens)
 	{
 		putErrmsg("Can't change congestion forecast horizon.", NULL);
 	}
+
+	oK(_forecastNeeded(1));
 }
 
 static void	manageAlarm(int tokenCount, char **tokens)
@@ -561,7 +686,7 @@ static void	manageAlarm(int tokenCount, char **tokens)
 		return;
 	}
 
-	sdr_begin_xn(sdr);
+	CHKVOID(sdr_begin_xn(sdr));
 	sdr_stage(sdr, (char *) &iondb, iondbObj, sizeof(IonDB));
 	if (iondb.alarmScript != 0)
 	{
@@ -581,6 +706,12 @@ static void	manageUsage(int tokenCount, char **tokens)
 	Sdr	sdr = getIonsdr();
 		OBJ_POINTER(IonDB, iondb);
 	char	buffer[128];
+	vast	heapOccupancy;
+	double	heapSpaceMBInUse;
+	vast	fileOccupancy;
+	double	fileSpaceMBInUse;
+	double	occupancyCeiling;	/*	In MBytes.		*/
+	double	maxForecastOccupancy;	/*	In MBytes.		*/
 
 	if (tokenCount != 2)
 	{
@@ -588,10 +719,19 @@ static void	manageUsage(int tokenCount, char **tokens)
 		return;
 	}
 
+	CHKVOID(sdr_begin_xn(sdr));
+	heapOccupancy = zco_get_heap_occupancy(sdr);
+	fileOccupancy = zco_get_file_occupancy(sdr);
+	heapSpaceMBInUse = heapOccupancy / 1000000;
+	fileSpaceMBInUse = fileOccupancy / 1000000;
 	GET_OBJ_POINTER(sdr, IonDB, iondb, getIonDbObject());
-	isprintf(buffer, sizeof buffer, "current = %ld  limit = %ld  max \
-forecast = %ld", iondb->currentOccupancy, iondb->occupancyCeiling,
-			iondb->maxForecastOccupancy);
+	occupancyCeiling = iondb->occupancyCeiling / 1000000;
+	maxForecastOccupancy = iondb->maxForecastOccupancy / 1000000;
+	sdr_exit_xn(sdr);
+	isprintf(buffer, sizeof buffer, "current heap %.2f MB, \
+current file space %.2f MB, limit %.2f MB, max forecast %.2f MB",
+			heapSpaceMBInUse, fileSpaceMBInUse, occupancyCeiling,
+			maxForecastOccupancy);
 	printText(buffer);
 }
 
@@ -615,6 +755,12 @@ static void	executeManage(int tokenCount, char **tokens)
 		return;
 	}
 
+	if (strcmp(tokens[1], "clocksync") == 0)
+	{
+		manageClockSync(tokenCount, tokens);
+		return;
+	}
+
 	if (strcmp(tokens[1], "production") == 0
 	|| strcmp(tokens[1], "prod") == 0)
 	{
@@ -763,7 +909,7 @@ static int	processLine(char *line, int lineLength)
 			return 0;
 
 		case 'v':
-			isprintf(buffer, sizeof buffer, "ION version %s.",
+			isprintf(buffer, sizeof buffer, "%s",
 					IONVERSIONNUMBER);
 			printText(buffer);
 			return 0;
@@ -998,11 +1144,10 @@ static int	runIonadmin(char *cmdFileName)
 	writeErrmsgMemos();
 	if (ionAttach() == 0)
 	{
-		oK(checkForCongestion());
-	}
-	else
-	{
-		printText("Can't check for congestion.");
+		if (_forecastNeeded(0))
+		{
+			oK(pseudoshell("ionwarn"));
+		}
 	}
 
 	printText("Stopping ionadmin.");
